package example;

/*
 * Bench.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import javax.robotics.engine.robots.DHRobot;
import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.Matrix;
import javax.robotics.vecmath.RMatrix4d;
import javax.robotics.vecmath.RVector;

public class Bench
{

    public static void main(String[] args)
    {
        int NTRY = 2000;
        RVector v = null;
        RMatrix4d T = null;
        Matrix J = null;

        final double[] qr =
            { Math.PI / 4.0, Math.PI / 4.0, Math.PI / 4.0, Math.PI / 4.0, Math.PI / 4.0,
                    Math.PI / 4 };
        final double[] qs =
            { Math.PI / 16.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

        final double[] PUMA560_data =
            { 0, 0, 0, 0, Math.PI / 2.0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.35, 0,
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0.4318, 0, 0, 0, 0, 17.4,
                    -0.3638, 0.006, 0.2275, 0.13, 0, 0, 0.524, 0, 0.539, 0, 0,
                    0, 0, 0, 0, 0, 0.15005, 0.0203, -Math.PI / 2.0, 0, 0, 0,
                    4.8, -0.0203, -0.0141, 0.07, 0.066, 0, 0, 0.086, 0, 0.0125,
                    0, 0, 0, 0, 0, 0, 0, 0.4318, 0.0, Math.PI / 2.0, 0, 0, 0,
                    0.82, 0, 0.019, 0, 0.0018, 0, 0, 0.0013, 0, 0.0018, 0, 0,
                    0, 0, 0, 0, 0, 0, 0.0, -Math.PI / 2.0, 0, 0, 0, 0.34, 0.0,
                    0.0, 0.0, 0.0003, 0.0, 0.0, 0.0004, 0.0, 0.0003, 0, 0, 0,
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.09, 0.0, 0.0, 0.032,
                    0.00015, 0.0, 0.0, 0.00015, 0.0, 0.00004, 0, 0, 0, 0, 0 };

        final Robot robot = new DHRobot(6, PUMA560_data);

        System.out.println("Begin compute Forward Kinematics\n");
        long startTime = System.nanoTime();
        for (int i = 1; i <= NTRY; i++)
        {
            robot.setQ(qr);
            T = robot.kine();
        }
        long estimatedTime = System.nanoTime() - startTime;
        System.out.printf("MilliSeconds %1$6.2f\n", (estimatedTime / 1e+6)
                / NTRY);

        System.out.println(T);
        System.out.println("end \n");

        System.out.println("Begin compute Inverse Kinematics\n");
        robot.setQ(qr);
        T = robot.kine();
        startTime = System.nanoTime();
        for (int i = 1; i <= NTRY; i++)
        {
            robot.setQ(qs);
            v = robot.invKine(T);
        }
        estimatedTime = System.nanoTime() - startTime;
        System.out.printf("MilliSeconds %1$6.2f\n", (estimatedTime / 1e+6)
                / NTRY);
        System.out.println(v);
        System.out.println("end \n");

        System.out.println("Begin compute Jacobian\n");
        startTime = System.nanoTime();
        for (int i = 1; i <= NTRY; i++)
        {
            robot.setQ(qr);
            J = robot.jacobian();
        }
        estimatedTime = System.nanoTime() - startTime;
        System.out.printf("MilliSeconds %1$6.2f\n", (estimatedTime / 1e+6)
                / NTRY);
        System.out.println(J);
        System.out.println("end \n");

        System.out.println("Begin compute Torque\n");
        robot.setQ(qr);
        T = robot.kine();
        startTime = System.nanoTime();
        for (int i = 1; i <= NTRY; i++)
        {
            v = robot.torque(qr, qr, qr);
        }
        estimatedTime = System.nanoTime() - startTime;
        System.out.printf("MilliSeconds %1$6.2f\n", (estimatedTime / 1e+6)
                / NTRY);
        System.out.println(v);
        System.out.println("end \n");

        System.out.println("Begin compute Acceleration\n");
        robot.setQ(qr);
        T = robot.kine();
        startTime = System.nanoTime();
        for (int i = 1; i <= NTRY; i++)
        {
            v = robot.acceleration(qr, qr, qr);
        }
        estimatedTime = System.nanoTime() - startTime;
        System.out.printf("MilliSeconds %1$6.2f\n", (estimatedTime / 1e+6)
                / NTRY);
        System.out.println(v);
        System.out.println("end \n");

    }

}
