package example;

/*
 * DemoClikConstraint.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.clik.CLIKR;
import javax.robotics.engine.clik.ClikJacobianPosInverse;
import javax.robotics.engine.clik.JointStrokeConstraint;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.simulator.ClikWithConstraintSimulation;
import javax.robotics.swing.DisplayPanel;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.apache.xmlbeans.XmlException;

import pathML.PathConfigurationType;

/**
 * CLIK simulator demonstration with joints constraint.
 * Section 3.7.4 of Modelling and Control of Robot Manipulators by Sciavicco,
 * Siciliano(2nd Edition)
 * 
 * @author Carmine Lia
 * 
 */
public class DemoClikConstraint extends JFrame implements ActionListener
{
    /**
     * 
     */
    private static final long serialVersionUID = 3256725099943049012L;
    private final String newLine =  System.getProperty("line.separator");
    
    private final class EndEffPath implements Trajectory
    {

        public short getID()
        {
            return 0;
        }

        public short getError()
        {
            return 0;
        }

        public PathConfigurationType getXmlHandler()
        {
            return null;
        }

        public void interpolate(double t, RVector p, RVector pp, RVector ppp)
        {
            if(t < tf - dt)
            {
                p.setElement(0, 0.25 * (1.0 - Math.cos(Math.PI * t)));
                p.setElement(1, 0.25 * (2.0 + Math.sin(Math.PI * t)));
                pp.setElement(0, 0.25 * Math.PI * Math.sin(Math.PI * t));
                pp.setElement(1, 0.25 * Math.PI * Math.cos(Math.PI * t));
            }
            else
            {
                p.setValues(0.0, 0.5);
                pp.setValues(0, 0);
            }

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp)
        {

        }

        public void interpolate(double t, Quaternion q, RVector3d w)
        {

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp, Quaternion q, RVector3d w)
        {

        }

        public double[] interpolate(double t)
        {
            return null;
        }

        public void interpolate(double t, RVector3d p, RVector3d pp, RVector3d ppp, Quaternion q, RVector3d w, RVector3d wp)
        {
            
        }

    }

    // robotModel
    private final Robot3D robot;
    // clik
    private final CLIKR clik;
    // matrix gain
    private final DMatrix Kp;
    // constraint
    private final JointStrokeConstraint constraint;
    // clik simulator
    private final ClikWithConstraintSimulation clikSim;
    
    static final float dt = 0.001f;
    static final float tf = 4.0f;
    private static final float dur = 5.50f;
    private static final int punti = Math.round(dur / dt) + 1;
  
    final ClassLoader cl = this.getClass().getClassLoader();
    
    private DemoClikConstraint(String title) throws XmlException, IOException, URISyntaxException
    {
        super(title);
        
        DisplayPanel display = new DisplayPanel("Simulation description");
        display.append("Closed Loop Inverse Kinematics demonstration for 3 links planar robot manipulator with joints constraint"+newLine);
        display.append("Section 3.7.4 of Modelling and Control of Robot Manipulators by Sciavicco, Siciliano(2nd Edition)");
       
        
        final double[] q0 =
            { Math.PI, -Math.PI / 2.0, -Math.PI / 2.0 };

        Kp = new DMatrix(500, 500);
        
        // create the robot and set the initial position
        robot = new DHRobot3D(cl.getResourceAsStream("example/conf/rr_book_chap3.xml"));
        robot.setQ(q0);

        // definizione del vincolo sul fine corsa dei giunti
        constraint = new JointStrokeConstraint(250, robot.getQMax(), robot
                .getQMin());
        clik = new ClikJacobianPosInverse(robot, Kp, 2);

        // create the end effector path
        final EndEffPath path = new EndEffPath();

        // create the simulator
        clikSim = new ClikWithConstraintSimulation(robot, clik, constraint, path,
                punti);
        
        // get the plots panel of simulator
        final MultiPlotPanel plotsPanel = clikSim.getPlotPanel();
        plotsPanel.setLayout(new GridLayout(3, 1));
       
        // get the robot panel of simulator
        final RobotPanel robotPanel = clikSim.getRobotPanel();
        
        // the side panel with plots an play sim button
        final JPanel p = new JPanel();
        p.setLayout(new BorderLayout());

        final JButton b = new JButton("Play simulation");
        b.addActionListener(this);

        p.add(plotsPanel, BorderLayout.CENTER);
        p.add(b, BorderLayout.SOUTH);

        setPreferredSize(new Dimension(1024, 768));
        getContentPane().add(robotPanel, BorderLayout.CENTER);
        getContentPane().add(p, BorderLayout.EAST);
        pack();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);

        // start the simulator
        clikSim.setEnable(true);

    }

    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
        new DemoClikConstraint(
                "CLIK for 3 links planar robot with joints stroke constraint");
    }

    // button action
    public void actionPerformed(ActionEvent e)
    {
        clikSim.play();
    }

}