package example;

/*
 * DemoRNE.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnElapsedTime;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.ode.DerivableFunction2;
import javax.robotics.ode.RungeKutta4th2;
import javax.robotics.ode.Solver2;
import javax.robotics.swing.DynamicsPanel;
import javax.robotics.swing.FileBrowserDialog;
import javax.robotics.swing.JointSliderPanel;
import javax.robotics.swing.KinematicsPanel;
import javax.robotics.swing.RobotFrame;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.apache.xmlbeans.XmlException;

public final class DemoRNE extends RobotFrame implements DerivableFunction2
{
    /**
     * 
     */
    private static final long serialVersionUID = 3258412824404439351L;

    private final class SimulationStep extends Behavior
    {
        private long delta;
        @Override
        public void initialize()
        {
            delta = Math.round(dt * 1000);
            wakeUp = new WakeupOnElapsedTime(delta);
            wakeupOn(wakeUp);
        }

        @Override
        public void processStimulus(Enumeration arg0)
        {
            wakeupOn(wakeUp);

            q = solver.getLastValuePosition();
            robot.setQ3DOnly(q);
            // q.printlnf("%10.4f");
            qp = solver.getLastValueVelocity();
            solver.step(q, qp);
        }

    }

    /**
     * 
     */
    private final KinematicsPanel kinePanel;
    private final DynamicsPanel dynPanel;
    // robotModel
    final Robot3D robot;
    final Solver2 solver;
    WakeupCriterion wakeUp;
    final SimulationStep simstep;

    // numero dei giunti del robotModel
    final int dof;

    RVector q0, q, qp0, qp;
    static final double dt = 45e-3;
    private final RVector3d Fext, Next;

    private final String newLine = System.getProperty("line.separator");
    private final ClassLoader cl = this.getClass().getClassLoader();
    
    private DemoRNE() throws XmlException, IOException, URISyntaxException
    {
        super();
              
        Fext = new RVector3d();
        Next = new RVector3d();

        final FileBrowserDialog browser = new FileBrowserDialog(
                "Robot Configuration File",
                "http://www.intermedia.sa.it/lia/jroboop/xml/puma560.xml");
        final URI file = browser.getFile();

        robot = (file == null) ? new DHRobot3D(cl
                .getResourceAsStream("example/conf/puma560.xml"))
                : new DHRobot3D(file);
         dof = robot.getDof();
        q0 = robot.getQ();
        qp0 = new RVector(dof);
        // robot.setGravity(new RVector3d());
        System.out.println("B" + newLine + robot.inertia(q0));
        // System.out.println(robot.torqueC(new RVector(1,1)));
        System.out.println(robot.torqueGravity());
        System.out.println("Gravity vector" + newLine + robot.getGravity());

        // init solver runge-kutta4 with initial state
        solver = new RungeKutta4th2(dt, q0, qp0, this);

        // create simulation step behavior
        simstep = new SimulationStep();
        simstep.setEnable(false);

        addRobotToDefaultConfiguration(robot, simstep);

        // create the panel
        JPanel p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

        JTabbedPane tabbedPane = new JTabbedPane();
        kinePanel = new KinematicsPanel(robot);
        dynPanel = new DynamicsPanel(robot, Fext, Next);
        tabbedPane.addTab("Kinematics", null, kinePanel, "Kinematics");
        tabbedPane.setSelectedIndex(0);

        if(robot.getRobotConfiguration().isDynamicsSet())
            tabbedPane.addTab("Dynamics", dynPanel);

        p.add(tabbedPane);
        if(robot.getRobotConfiguration().isDynamicsSet())
            p.add(createButtonPanel());
        else
            p.add(new JLabel("No dynamics data"));

        getContentPane().add(p, BorderLayout.EAST);
        setTitle(robot.getRobotConfiguration().getName());
        pack();
        setVisible(true);

        JFrame sliderWindow = new JFrame("Joints Window");
        sliderWindow.getContentPane().add(new JointSliderPanel(robot));
        sliderWindow.pack();
        sliderWindow.setVisible(true);

    }

    // implements the derivable function
    public RVector xDot2(RVector q, RVector qp)
    {
        return robot.acceleration(q, qp, new RVector(dof), Fext, Next);
    }

    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
        new DemoRNE();
    }

    private JPanel createButtonPanel()
    {
        final JButton b = new JButton("Play");

        final JPanel bp = new JPanel();
        bp.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createLineBorder(Color.gray), "Simulation"));

        final JButton b1 = new JButton("Init");
        b1
                .setToolTipText("Stops the simulation and sets the simulator to the current robot position");

        b1.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                simstep.setEnable(false);
                q0 = kinePanel.getJointPanel().getValue();
                qp0 = new RVector(dof);

                robot.setQ3D(q0);
                robot.setQp(qp0);

                solver.setInitialConditionPosition(q0);
                solver.setInitialConditionVelocity(qp0);
                solver.resetTime();

                b.setForeground(Color.BLACK);
                b.setText("Play");
            }
        });
        bp.add(b1);

        //
        b.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                simstep.setEnable(!simstep.getEnable());
                if(!simstep.getEnable())
                {
                    b.setForeground(Color.BLACK);
                    b.setText("Play");
                }
                else
                {
                    b.setForeground(Color.RED);
                    b.setText("Pause");
                }
            }
        });
        bp.add(b);
        return bp;
    }
}