package example;

/*
 * TestClik.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.clik.CLIKR;
import javax.robotics.engine.clik.ClikJacobianPosInverse;
import javax.robotics.engine.clik.ClikJacobianPosTranspose;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.simulator.CLIKRSimulation;
import javax.robotics.swing.DisplayPanel;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.xmlbeans.XmlException;

import pathML.PathConfigurationType;
/**
 * Section 3.7.4 of Modelling and Control of Robot Manipulators by Sciavicco,
 * Siciliano(2nd Edition)
 * 
 * @author Carmine Lia
 * 
 */
public final class TestClik extends JFrame implements ActionListener
{
    /**
     * 
     */
    private static final long serialVersionUID = 3256725099943049012L;
    final ClassLoader cl = this.getClass().getClassLoader();
    
    private class EndEffPath implements Trajectory
    {

        public short getID()
        {
            return 0;
        }

        public short getError()
        {
            return 0;
        }

        public PathConfigurationType getXmlHandler()
        {
            return null;
        }

        public void interpolate(double t, RVector p, RVector pp, RVector ppp)
        {
            if(t < tf - dt)
            {
                p.setElement(0, 0.25 * (1.0 - Math.cos(Math.PI * t)));
                p.setElement(1, 0.25 * (2.0 + Math.sin(Math.PI * t)));
                pp.setElement(0, 0.25 * Math.PI * Math.sin(Math.PI * t));
                pp.setElement(1, 0.25 * Math.PI * Math.cos(Math.PI * t));
            }
            else
            {
                p.setValues(0.0, 0.5);
                pp.setValues(0, 0);
            }

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp)
        {

        }

        public void interpolate(double t, Quaternion q, RVector3d w)
        {

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp, Quaternion q, RVector3d w)
        {

        }

        public double[] interpolate(double t)
        {
            return null;
        }

        public void interpolate(double t, RVector3d p, RVector3d pp, RVector3d ppp, Quaternion q, RVector3d w, RVector3d wp)
        {
            // TODO Auto-generated method stub
            
        }

    }

    // robotModel
    private final Robot3D robot;
    private final CLIKR clik;
    private final DMatrix Kp;
    private final EndEffPath path;
    private final CLIKRSimulation clikSim;
    
    static final float dt = 1e-3f;
    static final float tf = 4.0f;
    private static final float dur = 5.0f;
    private static final int punti = Math.round(dur / dt) + 1;
  
    private final String newLine =  System.getProperty("line.separator");

    
    private TestClik() throws XmlException, IOException, URISyntaxException
    {
        super();
        DisplayPanel display = new DisplayPanel("Simulation description");
        display.append("Closed Loop Inverse Kinematics demonstration for 3 links planar robot manipulator"+newLine);
        display.append("Section 3.7.4 of Modelling and Control of Robot Manipulators by Sciavicco, Siciliano(2nd Edition)");
        double q0[] =
            { Math.PI, -Math.PI / 2.0, -Math.PI / 2.0 };

        Kp = new DMatrix(500, 500);

        robot = new DHRobot3D(cl.getResourceAsStream("example/conf/rr_book_chap3.xml"));
        robot.setQ(q0);
        
        String[] possibleValues =
            { "inverse", "transpose" };
        String selectedValue = (String)JOptionPane.showInputDialog(null,
                "CLIK jacobian", "Choose the CLIK",
                JOptionPane.INFORMATION_MESSAGE, null, possibleValues,
                possibleValues[0]);
        
        if(selectedValue == null)
        		System.exit(1);
        
        if(selectedValue.equals("transpose"))
        {
            clik = new ClikJacobianPosTranspose(robot, Kp, 2);
            setTitle("CLIK for 3 links planar robot with Jacobian transpose");
        }
            
        else
        {
            clik = new ClikJacobianPosInverse(robot, Kp, 2);
            setTitle("CLIK for 3 links planar robot with Jacobian inverse");
        }
        
        path = new EndEffPath();

        clikSim = new CLIKRSimulation(robot, clik, path, punti);
        MultiPlotPanel plotsPanel = clikSim.getPlotPanel();
        plotsPanel.setLayout(new GridLayout(3, 1));
        RobotPanel robotPanel = clikSim.getRobotPanel();

        final JPanel p = new JPanel();
        p.setLayout(new BorderLayout());

        final JButton b = new JButton("Play simulation");
        b.addActionListener(this);

        p.add(plotsPanel, BorderLayout.CENTER);
        p.add(b, BorderLayout.SOUTH);

        setPreferredSize(new Dimension(1024, 768));
        getContentPane().add(robotPanel, BorderLayout.CENTER);
        getContentPane().add(p, BorderLayout.EAST);
        pack();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);

        clikSim.setEnable(true);

    }

    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
       new TestClik();
    }

    // button action
    public void actionPerformed(ActionEvent e)
    {
        clikSim.play();
    }
}