package example;

/*
 * TestClikConstraint.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.robotics.engine.clik.CLIKR;
import javax.robotics.engine.clik.ClikJacobianPosInverse;
import javax.robotics.engine.clik.ConstraintFunction;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.j3d.CollisionDetector;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.simulator.ClikWithConstraintSimulation;
import javax.robotics.swing.DisplayPanel;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

import org.apache.xmlbeans.XmlException;

import pathML.PathConfigurationType;

import com.sun.j3d.utils.geometry.Sphere;

/**
 * Solution of problem 3.20 of Modelling and Control of Robot Manipulators by
 * Sciavicco, Siciliano(2nd Edition)
 * 
 * @author Carmine Lia
 * 
 */
public final class TestClikConstraint extends JFrame implements ActionListener,
        ChangeListener
{
    // End Effector path
    private class EndEffPath implements Trajectory
    {

        public short getID()
        {
            return 0;
        }

        public short getError()
        {
            return 0;
        }

        public PathConfigurationType getXmlHandler()
        {
            return null;
        }

        public void interpolate(double t, RVector p, RVector pp, RVector ppp)
        {
            if(t < tf)
            {
                s = 0.5 * (1.0 - Math.cos(w * t));
                ds = 0.5 * w * Math.sin(w * t);
                p.setElement(0, pi[0] + s * (pf[0] - pi[0]));
                p.setElement(1, pi[1] + s * (pf[1] - pi[1]));
                pp.setElement(0, ds * (pf[0] - pi[0]));
                pp.setElement(1, ds * (pf[1] - pi[1]));
            }
            else
            {
                p.setElement(0, pf[0]);
                p.setElement(1, pf[1]);
                pp.setElement(0, 0);
                pp.setElement(1, 0);
            }
        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp)
        {

        }

        public void interpolate(double t, Quaternion q, RVector3d w)
        {

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp, Quaternion q, RVector3d w)
        {

        }

        public double[] interpolate(double t)
        {
            return null;
        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp, Quaternion q, RVector3d w, RVector3d wp)
        {
     
        }

    }
    // definition of constraint function
    private class ObstacleConstraint extends ConstraintFunction
    {
        double radius, diffx, diffy, a2, dmin, d;
        double a[] =
            { 0.5, 0.3, 0.3 };
        double s_opt, s;
        Point2d center, p_link;
        Point2d[] p;
        ObstacleConstraint(double k0, double radius, Point2d center)
        {
            super(k0);
            this.radius = radius;
            this.center = center;
            p = new Point2d[4];
            p_link = new Point2d();
        }

        /**
         * The function computes the minimum distance from obstacle
         */
        public double function(RVector q)
        {
            dmin = 1e+10;
            final double q12 = q.getElement(0) + q.getElement(1);
            final double q123 = q12 + q.getElement(2);
            final double c1 = Math.cos(q.getElement(0));
            final double s1 = Math.sin(q.getElement(0));
            final double c12 = Math.cos(q12);
            final double s12 = Math.sin(q12);
            final double c123 = Math.cos(q123);
            final double s123 = Math.sin(q123);
            p[0] = new Point2d();
            p[1] = new Point2d(a[0] * c1, a[0] * s1);
            p[2] = new Point2d(a[0] * c1 + a[1] * c12, a[0] * s1 + a[1] * s12);
            p[3] = new Point2d(a[0] * c1 + a[1] * c12 + a[2] + c123, a[0] * s1
                    + a[1] * s12 + a[2] + s123);

            for (int i = 1; i <= 3; i++)
            {
                diffx = p[i].x - p[i - 1].x;
                diffy = p[i].y - p[i - 1].y;
                a2 = diffx * diffx + diffy * diffy;
                s_opt = -((p[i - 1].x - center.x) * diffx + (p[i - 1].y - center.y)
                        * diffy)
                        / a2;
                s = Math.max(0.0, Math.min(1.0, s_opt));
                p_link.x = p[i - 1].x + s * diffx;
                p_link.y = p[i - 1].y + s * diffy;

                d = p_link.distance(center);

                if(d < dmin)
                    dmin = d;
            }
            dmin -= radius;
            return dmin;
        }

    }

    /**
     * 
     */
    private static final long serialVersionUID = 3256725099943049012L;

    private final ClassLoader cl = this.getClass().getClassLoader();
   
    private final String newLine =  System.getProperty("line.separator");
    
    // robotModel
    private Robot3D robot;
    private CLIKR clik;
    private DMatrix Kp;
    // constraint
    private final ObstacleConstraint constraint;
    // simulator
    private final ClikWithConstraintSimulation clikSim;
    // collision detector
    private CollisionDetector cd;
    // swing
    private JCheckBox enableCollision;

    // parametri per la simulazione
    private static final float dt = 1e-3f;
    private static final float dur = 2.5f;
    static final float tf = 2.0f;
    private static final int punti = Math.round(dur / dt) + 1;

    // path
    // initial position
    final double[] pi =
        { 0.8, 0.2 };
    // final position
    final double[] pf =
        { 0.8, -0.2 };
    double s, ds, w;

    private TestClikConstraint() throws XmlException, IOException,
            URISyntaxException
    {

        super();
        
        DisplayPanel display = new DisplayPanel("Simulation description");
        display.append("Closed Loop Inverse Kinematics demonstration for 3 links planar robot manipulator with obstacle"+newLine);
        display.append("Solution of problem 3.20 of Modelling and Control of Robot Manipulators by Sciavicco, Siciliano(2nd Edition)");
       
        w = Math.PI / tf;

        final double q[] =
            { 0.5147, -1.5517, 1.8224 };

        Kp = new DMatrix(500, 500);

        // create the robot an set the initial position
        robot = new DHRobot3D(cl.getResourceAsStream(
                "example/conf/rr_book_chap3_problem3.20.xml"));
        robot.setQ(q);

        // System.out.println(robot.kine());

        clik = new ClikJacobianPosInverse(robot, Kp, 2);

        String[] possibleValues =
            { "costrained", "uncostrained" };
        String selectedValue = (String) JOptionPane.showInputDialog(null,
                 "Choose the solution", "CLIK with obstacle costraint",
                JOptionPane.INFORMATION_MESSAGE, null, possibleValues,
                possibleValues[0]);
       
        if(selectedValue == null)
            System.exit(0);
        
        if(selectedValue.equals("costrained"))
        {
            // define the obstacle
            // set the gain of constraint velocity function
            constraint = new ObstacleConstraint(35.0, 1.0, new Point2d(0.3, 0.0));
            setTitle("CLIK for 3 links planar robot with obstacle, constrained solution");
        }
        else
        {
            // define the obstacle
            // set the gain of constraint velocity function
            constraint = new ObstacleConstraint(0.0, 1.0, new Point2d(0.3, 0.0));
            setTitle("CLIK for 3 links planar robot with obstacle, unconstrained solution");
        }

        // create the path
        final EndEffPath path = new EndEffPath();

        // create the simulator
        clikSim = new ClikWithConstraintSimulation(robot, clik, constraint,
                path, punti);

        // get the plots panel from simulator
        MultiPlotPanel plotsPanel = clikSim.getPlotPanel();
        plotsPanel.setLayout(new GridLayout(3, 1));

        // get the robot 3D panel from simulator
        RobotPanel robotPanel = clikSim.getRobotPanel();

        // create the obstacle in 3D scene as sphere
        Sphere obstacle = new Sphere(0.08f);
        Transform3D translate = new Transform3D();
        translate.setTranslation(new Vector3d(0.30, 0.0, 0.0));
        TransformGroup tg = new TransformGroup(translate);

        // adding the collision detector
        cd = new CollisionDetector(obstacle);
        cd.setSchedulingBounds(new BoundingSphere(new Point3d(0.0, 0.0, 0.0),
                100.0));
        tg.addChild(cd);

        cd.setEnable(true);
        // add the obstacle to 3D scene
        tg.addChild(obstacle);
        BranchGroup bg = new BranchGroup();
        bg.addChild(tg);
        robotPanel.get3DRootBranch().addChild(bg);

        // the side pane with plots and buttons
        final JPanel p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
        p.setBackground(Color.WHITE);

        final JButton b = new JButton("Play simulation");
        b.addActionListener(this);

        enableCollision = new JCheckBox("Enable collision detection");
        enableCollision.setSelected(true);
        enableCollision.addChangeListener(this);
        enableCollision.setBackground(Color.WHITE);

        p.add(plotsPanel);
        p.add(enableCollision);
        p.add(b);
        
        setPreferredSize(new Dimension(1024, 768));
        getContentPane().add(robotPanel, BorderLayout.CENTER);
        getContentPane().add(p, BorderLayout.EAST);
        pack();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);

        // start the simulator
        clikSim.setEnable(true);
    }

    public static void main(String[] args) throws XmlException, IOException,
            URISyntaxException
    {
        new TestClikConstraint();
    }

    public void actionPerformed(ActionEvent e)
    {
        // play simulation
        clikSim.play();
    }

    public void stateChanged(ChangeEvent e)
    {
        // toggle collision detection 
        cd.setEnable(!cd.getEnable());
    }

}