package example;

/*
 * TestClikScara.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.clik.CLIK;
import javax.robotics.engine.clik.ClikJacobianInverse;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.simulator.CLIKSimulation;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.apache.xmlbeans.XmlException;

import pathML.PathConfigurationType;

/**
 * Solution of problem 3.21 of Modelling and Control of Robot Manipulators by
 * Sciavicco, Siciliano(2nd Edition)
 * 
 * @author Carmine Lia
 * 
 */
public final class TestClikScara extends JFrame implements ActionListener
{
    private class EndEffPath implements Trajectory
    {

        public short getID()
        {
            return 0;
        }

        public short getError()
        {
            return 0;
        }

        public PathConfigurationType getXmlHandler()
        {
            return null;
        }

        public void interpolate(double t, RVector p, RVector pp, RVector ppp)
        {

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp)
        {

        }

        public void interpolate(double t, Quaternion q, RVector3d w)
        {

        }

        public void interpolate(double t, RVector3d p, RVector3d pp,
                RVector3d ppp, Quaternion q, RVector3d w)
        {
            if(t < tf)
            {
                // s = 0.5*(1 - cos((pi/t_f)*T_1));
                s = 0.5 * (1.0 - Math.cos((Math.PI / tf) * t));
                // ds = 0.5*(pi/t_f)*sin((pi/t_f)*T_1);
                ds = 0.5 * (Math.PI / tf) * Math.sin((Math.PI / tf) * t);
                p.x = pi.x + s * (pf.x - pi.x);
                p.y = pi.y + s * (pf.y - pi.y);
                p.z = pi.z + s * (pf.z - pi.z);
                angle = angle_initial + s * (angle_final - angle_initial);

                pp.x = ds * (pf.x - pi.x);
                pp.y = ds * (pf.y - pi.y);
                pp.z = ds * (pf.z - pi.z);

                w.x = 0;
                w.y = 0;
                w.z = ds * (angle_final - angle_initial);
            }
            else
            {
                p = pf;
                angle = angle_final;

                pp.x = 0;
                pp.y = 0;
                pp.z = 0;

                w.x = 0;
                w.y = 0;
                w.z = 0;
            }
            angle /= 2.0;
            q.x = 0;
            q.y = 0;
            q.z = Math.sin(angle);
            q.w = Math.cos(angle);

        }

        public double[] interpolate(double t)
        {
            return null;
        }

        public void interpolate(double t, RVector3d p, RVector3d pp, RVector3d ppp, Quaternion q, RVector3d w, RVector3d wp)
        {
        }

    }

    /**
     * 
     */
    private static final long serialVersionUID = 3256725099943049012L;
    private final ClassLoader cl = this.getClass().getClassLoader();
  
    // robotModel
    private final Robot3D robot;
    private final CLIK clik;
    private final DMatrix Kp, Kd;
    private CLIKSimulation clikSim;
    
    // parametri per la simulazione
    int delta;
    private static final float dt = 1e-3f;
    private static final float dur = 2.5f;
    static final float tf = 2.0f;
    private static final int punti =  Math.round(dur / dt) + 1;

    
    // path
    double s, ds;
    // initial position
    static final RVector3d pi = new RVector3d(0.7, 0.0, 0.0);
    // final position
    static final RVector3d pf = new RVector3d(0.0, 0.8, 0.5);
    static final double angle_initial = 0.0;
    static final double angle_final = Math.PI / 2.0;
    double angle;

    private TestClikScara() throws XmlException, IOException, URISyntaxException
    {

        super();

        final double q[] =
            { 0.79539883018414, -1.59079766036829, 0.0, 0.79539883018414 };

        Kp = new DMatrix(500, 500, 500);
        Kd = new DMatrix(100, 100, 100);

        robot = new DHRobot3D(cl.getResourceAsStream("example/conf/scara_nodynamics.xml"));
        robot.setQ(q);

        clik = new ClikJacobianInverse(robot, Kp, Kd);
        
        final EndEffPath path = new EndEffPath();
        
        clikSim = new CLIKSimulation(robot, clik, path, punti);
        
        final MultiPlotPanel plotsPanel = clikSim.getPlotPanel();
        plotsPanel.setLayout(new GridLayout(4,1));
        final RobotPanel robotPanel = clikSim.getRobotPanel();
        
        final JPanel p = new JPanel();
        p.setLayout(new BorderLayout());
       
        final JButton b = new JButton("Play simulation");
        b.addActionListener(this);
         
        p.add(plotsPanel, BorderLayout.CENTER);
        p.add(b, BorderLayout.SOUTH);
        
        setPreferredSize(new Dimension(1024, 768));
        getContentPane().add(robotPanel, BorderLayout.CENTER);
        getContentPane().add(p, BorderLayout.EAST);
        pack();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);
       
        // start the simulator
        clikSim.setEnable(true);
        setTitle("CLIK for " + robot.getRobotConfiguration().getName());

    }

    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
        new TestClikScara();
    }

    public void actionPerformed(ActionEvent e)
    {
        // play the simulation
        clikSim.play();
    }

}