package example;

/*
 * TestRNE2linkRR.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnElapsedTime;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.ode.DerivableFunction2;
import javax.robotics.ode.RungeKutta4th2;
import javax.robotics.ode.Solver2;
import javax.robotics.swing.RobotFrame;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;
import javax.swing.JButton;
import javax.swing.JFrame;

public class TestRNE2linkRR extends RobotFrame implements DerivableFunction2,
        ActionListener
{
    private class SimulationStep extends Behavior
    {

        int delta;

        @Override
        public void initialize()
        {
            delta = (int) Math.round(dt * 1000);
            wakeUp = new WakeupOnElapsedTime(delta);
            wakeupOn(wakeUp);
        }

        @Override
        public void processStimulus(Enumeration arg0)
        {
            time = solver.getTime();
            wakeupOn(wakeUp);

            q = solver.getLastValuePosition();
            robot.setQ3DOnly(q);
       //     q.printlnf("%10.4f");
            qp = solver.getLastValueVelocity();
            solver.step(q, qp);
        }

    }

    /**
     * 
     */
    private static final long serialVersionUID = 3906651907677567288L;
    JFrame sliderWindow;
    JButton b;

    // robotModel
    Robot3D robot;
    Solver2 solver;
    WakeupCriterion wakeUp;
    SimulationStep simstep;

    RVector q, qp;
    double time = 0.0;
    static final double dt = 50e-3;

    TestRNE2linkRR(String title)
    {
        super(title);

        double[] conf =
                        {
                          0, 0, 0, .50, 0, 0, 0, 0, 55.0, -0.2272727, 0, 0, 0, 0, 0, 0, 0, 10.294091, 0.01, 100.0, 0.002, 0, 0,
                          0, 0, 0, .50, 0, 0, 0, 0, 50.0, -0.25,      0, 0, 0, 0, 0, 0, 0, 10.0,      0.01, 100.0, 0.002, 0, 0
                        };

        q = new RVector(Math.PI/1.85, -Math.PI/8);
        qp = new RVector(2);
        robot = new DHRobot3D(2, conf);
        robot.setQ3D(q);
        // set the gravity vector
        robot.setGravity(new RVector3d(0.0, 9.81, 0.0));
        System.out.println("Gravity vector:\n" + robot.getGravity());
        // create simulation step behavior
        simstep = new SimulationStep();
        simstep.setEnable(false);

        addRobotToDefaultConfiguration(robot, simstep);

        // init solver runge-kutta4 with initial state
        solver = new RungeKutta4th2(dt, new RVector(q), new RVector(qp), this);

        b = new JButton("Start/Stop simulation");
        b.addActionListener(this);

        add(b, BorderLayout.SOUTH);
        setSize(600, 600);
        setVisible(true);
    }

    // implements the derivable function
    public RVector xDot2(RVector q, RVector qp)
    {
        return robot.acceleration(q, qp, new RVector(2));
    }

    public static void main(String[] args)
    {
        new TestRNE2linkRR("Two revolute links planar robot");
    }

    public void actionPerformed(ActionEvent e)
    {

        simstep.setEnable(!simstep.getEnable());

    }

}