/* 
 * C++ library for JRoboOp
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 *  compile as shared library
 *  GNU Compiler command for linux:
 *  g++ -shared -I$ROBOOP_INCLUDE/roboop -I-I$JAVA_LINUX_PATH/include -I$JAVA_LINUX_PATH/include/linux JRoboOp.cpp -L$ROBOOP_LIB -o libJRoboOp.so -lroboop -lnewmat -lm
 
 *  GNU Compiler command for Win:
 	g++ -c JRoboOp.cpp -o JRoboOp.o -I%ROBOOP_INCLUDE% -L%ROBOOP_LIB% -I%JAVA_WIN_PATH%/include -I%JAVA_WIN_PATH%/include/win32
	dlltool -v -A -z JRoboOp.def -e exports.o -l JRoboOp.lib JRoboOp.o
	g++ -shared exports.o JRoboOp.o -o JRoboOp.dll %ROBOOP_LIB%/libroboop.a %ROBOOP_LIB%/libnewmat.a
 */
 
#include "JRoboOp.h"
#include <robot.h>
#include <quaternion.h>
#include <trajectory.h>
//#include <clik.h>
#include <controller.h>
#include <vector>

using namespace std;

short ID = -1;

short N = 1;

vector<Robot> robot(N);
//vector<Clik> clik(N);
vector<Spl_path> splPath(N);
vector<Spl_Quaternion> splQuat(N);
vector<Computed_torque_method> ctm(N);
vector<Robust_control> robust(N);
vector<Proportional_Derivative> pd(N);
vector<PDGravityComp> pdg(N);
vector<Resolved_acc> ra(N);
vector<Gravity_comp> gcomp(N);
vector<Impedance> imp(N);

vector<int> dof(N, 0);
vector<int> adof(N, 0);
vector<bool> converge(N, false);
vector<short> jerror(N, 0);

JNIEXPORT jshort JNICALL
Java_javax_robotics_engine_JRoboOp_setID(JNIEnv * env, jobject obj)
{
	if(ID >= N-1)
	{
		robot.resize(robot.size()+1);
		//clik.resize(clik.size()+1);
		splPath.resize(splPath.size()+1);
		splQuat.resize(splQuat.size()+1);
		ctm.resize(ctm.size()+1);
		robust.resize(robust.size()+1);
		pd.resize(pd.size()+1);
		pdg.resize(pdg.size()+1);
		ra.resize(ra.size()+1);
		gcomp.resize(gcomp.size()+1);
		imp.resize(imp.size()+1);
		dof.resize(dof.size()+1);
		adof.resize(dof.size()+1);
		converge.resize(converge.size()+1);
		jerror.resize(jerror.size()+1);
	}
	return ++ID;
	
}

JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_getError(JNIEnv *env, jobject obj, jshort id)
{
	short val = jerror[id];
	jerror[id] = 0;
	return val;	
}

JNIEXPORT jboolean JNICALL 
Java_javax_robotics_engine_JRoboOp_getConverge(JNIEnv *env, jobject obj, jshort id)
{
	bool val = converge[id];
	converge[id] = false;
	return val;
}

/*
 * 
 * Constructors
 * 
 */

JNIEXPORT void JNICALL
Java_javax_robotics_engine_JRoboOp_createRobot__S(JNIEnv *env, jobject obj, jshort id)
{
  robot[id] = Robot();
  dof[id] = 1;
  adof[id] = robot[id].get_available_dof();
  return;
}

JNIEXPORT void JNICALL
Java_javax_robotics_engine_JRoboOp_createRobot__S_3DII(JNIEnv *env, jobject obj, jshort id, jdoubleArray jinitRobot, jint jrows, jint jcols)
{
	jdouble *cinitRobot;
	Matrix initRobot(jrows, jcols);
	cinitRobot = env->GetDoubleArrayElements(jinitRobot, NULL);
	initRobot << cinitRobot;
	robot[id] = Robot(initRobot);
	env->ReleaseDoubleArrayElements(jinitRobot, cinitRobot,0);
	dof[id] = robot[id].get_dof();
	adof[id] = robot[id].get_available_dof();
	initRobot.Release();
	return;
}

JNIEXPORT void JNICALL
Java_javax_robotics_engine_JRoboOp_createRobot__S_3DII_3DII(JNIEnv *env, jobject obj, jshort id, jdoubleArray jinitRobot, jint jrowsR, jint jcolsR, jdoubleArray jinitMotor, jint jrowsM, jint jcolsM)
{
	jdouble *cinitRobot, *cinitMotor;
	Matrix initRobot(jrowsR, jcolsR), initMotor(jrowsM, jcolsM);
	cinitRobot = env->GetDoubleArrayElements(jinitRobot, NULL);
	cinitMotor = env->GetDoubleArrayElements(jinitMotor, NULL);
        initRobot << cinitRobot;
	initMotor << cinitMotor;
	robot[id] = Robot(initRobot, initMotor);
	env->ReleaseDoubleArrayElements(jinitRobot,cinitRobot,0);
	env->ReleaseDoubleArrayElements(jinitMotor,cinitMotor,0);
	initRobot.Release();
	initMotor.Release();
	dof[id] = robot[id].get_dof();
	adof[id] = robot[id].get_available_dof();
	return;
}

JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createRobot__SLjava_lang_String_2Ljava_lang_String_2(JNIEnv *env, jobject obj, jshort id, jstring jfileName, jstring jrobotName)
{
	char cfileName[50], crobotName[50];
	int fileNameLen = env->GetStringLength(jfileName);
	int robotNameLen = env->GetStringLength(jrobotName);
	env->GetStringUTFRegion(jfileName, 0, fileNameLen, cfileName);
	env->GetStringUTFRegion(jrobotName, 0, robotNameLen, crobotName);
	robot[id] = Robot(cfileName, crobotName);
	dof[id] = robot[id].get_dof();
	adof[id] = robot[id].get_available_dof();
	return;
}

// ------------------------ Robot Variables --------------------------

JNIEXPORT jint JNICALL 
Java_javax_robotics_engine_JRoboOp_getDof(JNIEnv *env, jobject obj, jshort id)
{
	return dof[id];
}

JNIEXPORT jint JNICALL 
Java_javax_robotics_engine_JRoboOp_getAvDof(JNIEnv *env, jobject obj, jshort id)
{
	return adof[id];
}
//------------------------- Joint Variables ---------------------------
// -- Getters
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getQ__S(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cq;
	jdoubleArray jq;
	Matrix q = robot[id].get_q();
	cq = q.data();
	jq = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jq, 0 ,dof[id], cq);
	q.Release();
	return jq;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getQMin(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cq;
	jdoubleArray jq;
	Matrix q = robot[id].get_q_min();
	int size = q.size();
//cout << "size " << size << endl;
	cq = q.data();
	jq = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jq, 0 ,size, cq);
	q.Release();
	return jq;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getQMax(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cq;
	jdoubleArray jq;
	Matrix q = robot[id].get_q_max();
	int size = q.size();
	cq = q.data();
	jq = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jq, 0 ,size, cq);
	q.Release();
	return jq;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getQOffset(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cq;
	jdoubleArray jq;
	Matrix q = robot[id].get_q_offset();
	int size = q.size();
	cq = q.data();
	jq = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jq, 0, size, cq);
	q.Release();
	return jq;
}

JNIEXPORT jdouble JNICALL 
Java_javax_robotics_engine_JRoboOp_getQ__SI(JNIEnv *env, jobject obj, jshort id, jint i)
{
	return robot[id].get_q(i);
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getQp(JNIEnv *env, jobject obj, jshort id)
{
	
	jdouble *cqp;
	jdoubleArray jqp;
	Matrix qp = robot[id].get_qp();
	int size = qp.size();
	cqp = qp.data();
	jqp = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jqp, 0, size, cqp);
	qp.Release();
	return jqp;
}

/* not yet implemented in roboop
JNIEXPORT jdouble JNICALL 
Java_javax_robotics_engine_JRoboOp_getQp__SI(JNIEnv *env, jobject obj, jshort id, jint i)
{
	return robot[id].get_qp(i);
}
*/

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getQpp(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cqpp;
	jdoubleArray jqpp;
	Matrix qpp = robot[id].get_qpp();
	int size = qpp.size();
	cqpp = qpp.data();
	jqpp = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jqpp, 0, size, cqpp);
	qpp.Release();
	return jqpp;
}

/* not yet implemented in roboop
JNIEXPORT jdouble JNICALL 
Java_javax_robotics_engine_JRoboOp_getQpp__SI(JNIEnv *env, jobject obj, jshort id, jint i)
{
	return robot[id].get_qpp(i);
}
*/

//-- Setters
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setQ__S_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq)
{
	jdouble *cq;
	ColumnVector q = ColumnVector(dof[id]);
	cq = env->GetDoubleArrayElements(jq, NULL);
	q << cq;
	robot[id].set_q(q);
	env->ReleaseDoubleArrayElements(jq,cq,0);
	q.Release();
	return;
}

JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setQ__SDI(JNIEnv *env, jobject obj, jshort id, jdouble q, jint i)
 {
	robot[id].set_q(q,i);
	return;
}

JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setQp(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq)
{
	jdouble *cq;
	ColumnVector q = ColumnVector(dof[id]);
	cq = env->GetDoubleArrayElements(jq, NULL);
	q << cq;
	robot[id].set_qp(q);
	env->ReleaseDoubleArrayElements(jq,cq,0);
	q.Release();
	return;
}

/* not yet implemented in roboop
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setQp__SDI(JNIEnv *env, jobject obj, jshort id, jdouble q, jint i)
 {
	robot[id].set_qp(q,i);
	return;
}
*/

JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setQpp(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq)
{
	jdouble *cq;
	ColumnVector q = ColumnVector(dof[id]);
	cq = env->GetDoubleArrayElements(jq, NULL);
	q << cq;
	robot[id].set_qpp(q);
	env->ReleaseDoubleArrayElements(jq,cq,0);
	q.Release();
	return;
}
/* not yet implemented in roboop
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setQpp__SDI(JNIEnv *env, jobject obj, jshort id, jdouble q, jint i)
 {
	robot[id].set_qpp(q,i);
	return;
}
*/

// ------------------------ Robot Kinematics --------------------------------

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_invKine__S_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jTobj)
{
	jdouble *cTobj;
	jdouble *cq;
	Matrix Tobj = Matrix(4,4);
	//ColumnVector q = ColumnVector(dof[id]);
	cTobj = env->GetDoubleArrayElements(jTobj, NULL);
	Tobj << cTobj;
cout << Tobj << endl;
	ColumnVector q = robot[id].inv_kin(Tobj);
cout << q << endl;
	int size = q.size();
	cq = q.data();
	jdoubleArray jq = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jq, 0, size, cq);
	env->ReleaseDoubleArrayElements(jTobj, cTobj,0);
	Tobj.Release();
	q.Release();
	return jq;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_invKine__S_3DII(JNIEnv *env, jobject obj, jshort id, jdoubleArray jTobj, jint mj, jint endlink)
{
	jdouble *cTobj;
	jdouble *cq;
	Matrix Tobj = Matrix(4,4);
	//ColumnVector q = ColumnVector(dof[id]);
	cTobj = env->GetDoubleArrayElements(jTobj, NULL);
	Tobj << cTobj;
	
	bool conv;
	ColumnVector q = robot[id].inv_kin(Tobj, mj, endlink, conv);
	int size = q.size();
	converge[id] = conv;
	cq = q.data();
	jdoubleArray jq = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jq, 0, size, cq);
	env->ReleaseDoubleArrayElements(jTobj, cTobj,0);
	Tobj.Release();
	q.Release();
	return jq;
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_jacobian__S(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cjaco;
	Matrix jaco = robot[id].jacobian();
	int size = jaco.size();
//cout << jaco << endl;
	cjaco = jaco.data();
	jdoubleArray jjaco = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jjaco, 0, size, cjaco);
	jaco.Release();
	return jjaco;
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_jacobian__SI(JNIEnv *env, jobject obj, jshort id, jint i)
{
	jdouble *cjaco;
	Matrix jaco = robot[id].jacobian(i);
	//int size = jaco.size();
//cout << jaco << endl;
	cjaco = jaco.data();
	jdoubleArray jjaco = env->NewDoubleArray(36);
	env->SetDoubleArrayRegion(jjaco, 0, 36, cjaco);
	jaco.Release();
	return jjaco;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_jacobianDot__S(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cjaco;
	Matrix jaco = robot[id].jacobian_dot();
	int size = jaco.size();
//cout << jaco << endl; 
	cjaco = jaco.data();
	jdoubleArray jjaco = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jjaco, 0, size, cjaco);
	jaco.Release();
	return jjaco;
}
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_jacobianDot__SI(JNIEnv *env, jobject obj, jshort id, jint i)
{
	jdouble *cjaco;
	Matrix jaco = robot[id].jacobian_dot(i);
	int size = jaco.size();
//cout << jaco << endl; 
	cjaco = jaco.data();
	jdoubleArray jjaco = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jjaco, 0, size, cjaco);
	jaco.Release();
	return jjaco;
}
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_jacobianDLSinv(JNIEnv *env, jobject obj, jshort id, jdouble eps, jdouble lambdaMax)
{
	jdouble *cjaco;
	Matrix jaco = robot[id].jacobian_DLS_inv(eps, lambdaMax);
	int size = jaco.size();
//cout << jaco << endl;
	cjaco = jaco.data();
	jdoubleArray jjaco = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jjaco, 0, size, cjaco);
	jaco.Release();
	return jjaco;
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_kine__S(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *ctransform;
	Matrix transform = robot[id].kine();
//cout << transform << endl;	
	int size = transform.size();
	ctransform = transform.data();
	jdoubleArray jtransform = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jtransform, 0, size, ctransform);
	transform.Release();
	return jtransform;
}  
JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_kine__SI(JNIEnv *env, jobject obj,  jshort id, jint lastJoint)
{
	jdouble *ctransform;
	Matrix transform = robot[id].kine(lastJoint);
//cout << transform << endl;
	int size = transform.size();
	ctransform = transform.data();
	jdoubleArray jtransform = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jtransform, 0, size, ctransform);
	return jtransform;
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_dTdqi(JNIEnv *env, jobject obj, jshort id, jint i)
{
	jdouble *ctransform;
	Matrix transform = robot[id].dTdqi(i);
//cout << transform << endl;
	ctransform = transform.data();
	jdoubleArray jtransform = env->NewDoubleArray(16);
	env->SetDoubleArrayRegion(jtransform, 0, 16, ctransform);
	return jtransform;
}
// ------------------ Robot Dynamics ----------------------------------
JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_acceleration__S_3D_3D_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq, jdoubleArray jqp, jdoubleArray jtau)
{
	jdouble *cq, *cqp, *ctau;
	jdouble *cacceleration;
	
	ColumnVector q = ColumnVector(dof[id]);
	ColumnVector qp = ColumnVector(dof[id]);
	ColumnVector tau = ColumnVector(dof[id]);
	
	cq = env->GetDoubleArrayElements(jq, NULL);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	ctau = env->GetDoubleArrayElements(jtau, NULL);
	
	q << cq;
	qp << cqp;
	tau << ctau;
	
	ColumnVector acceleration = robot[id].acceleration(q, qp, tau);
	cacceleration = acceleration.data();

//cout << acceleration << endl;
	
	jdoubleArray jacceleration = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jacceleration, 0, dof[id], cacceleration);
	return jacceleration;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_acceleration__S_3D_3D_3D_3D_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq, jdoubleArray jqp, jdoubleArray jtauCmd, jdoubleArray jFext, jdoubleArray jNext)
{
	jdouble *cq, *cqp, *ctauCmd, *cFext, *cNext;
	jdouble *cacceleration;
	
	ColumnVector q = ColumnVector(dof[id]);
	ColumnVector qp = ColumnVector(dof[id]);
	ColumnVector tauCmd = ColumnVector(dof[id]);
	ColumnVector Fext = ColumnVector(3);
	ColumnVector Next = ColumnVector(3);
	
	cq = env->GetDoubleArrayElements(jq, NULL);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	ctauCmd = env->GetDoubleArrayElements(jtauCmd, NULL);
	cFext = env->GetDoubleArrayElements(jFext, NULL);
	cNext = env->GetDoubleArrayElements(jNext, NULL);
	
	q << cq;
	qp << cqp;
	tauCmd << ctauCmd;
	Fext << cFext;
	Next << cNext;
	
	ColumnVector acceleration = robot[id].acceleration(q, qp, tauCmd, Fext, Next);
	cacceleration = acceleration.data();

//cout << acceleration << endl;

	jdoubleArray jacceleration = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jacceleration, 0, dof[id], cacceleration);
	return jacceleration;
}


JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_torque__S_3D_3D_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq, jdoubleArray jqp, jdoubleArray jqpp)
{
	jdouble *cq, *cqp, *cqpp;
	jdouble *ctorque;
	
	ColumnVector q = ColumnVector(dof[id]);
	ColumnVector qp = ColumnVector(dof[id]);
	ColumnVector qpp = ColumnVector(dof[id]);
	
	cq = env->GetDoubleArrayElements(jq, NULL);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	cqpp = env->GetDoubleArrayElements(jqpp, NULL);
	
	q << cq;
	qp << cqp;
	qpp << cqpp;
	ColumnVector torque = robot[id].torque(q, qp, qpp);
//cout << torque << endl;
	ctorque = torque.data();
	jdoubleArray jtorque = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jtorque, 0, dof[id], ctorque);
	return jtorque;	
}

JNIEXPORT jdoubleArray
JNICALL Java_javax_robotics_engine_JRoboOp_torque__S_3D_3D_3D_3D_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq, jdoubleArray jqp, jdoubleArray jqpp, jdoubleArray jFext, jdoubleArray jNext)
{
	jdouble *cq, *cqp, *cqpp, *cFext, *cNext;
	jdouble *ctorque;
	
	ColumnVector q = ColumnVector(dof[id]);
	ColumnVector qp = ColumnVector(dof[id]);
	ColumnVector qpp = ColumnVector(dof[id]);
	ColumnVector Fext = ColumnVector(3);
	ColumnVector Next = ColumnVector(3);
	
	cq = env->GetDoubleArrayElements(jq, NULL);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	cqpp = env->GetDoubleArrayElements(jqpp, NULL);
	cFext = env->GetDoubleArrayElements(jFext, NULL);
	cNext = env->GetDoubleArrayElements(jNext, NULL);
	
	q << cq;
	qp << cqp;
	qpp << cqpp;
	Fext << cFext;
	Next << cNext;
	
	ColumnVector torque = robot[id].torque(q, qp, qpp, Fext, Next);
//cout << torque << endl;	
	ctorque = torque.data();
	
	jdoubleArray jtorque = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jtorque, 0, dof[id], ctorque);
	return jtorque;	
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_torqueGravity(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *ctorque;
	
	ColumnVector torque = robot[id].G();
	ctorque = torque.data();
//cout << torque << endl;	
	
	jdoubleArray jtorque = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jtorque, 0, dof[id], ctorque);
	return jtorque;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_torqueCoriolis(JNIEnv *env, jobject obj, jshort id, jdoubleArray jqp)
{
	jdouble *cqp;
	jdouble *ctorque;
	
	ColumnVector qp = ColumnVector(dof[id]);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	qp << cqp;
		
	ColumnVector torque = robot[id].C(qp);
	ctorque = torque.data();
//cout << torque << endl;	
	
	jdoubleArray jtorque = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jtorque, 0, dof[id], ctorque);
	return jtorque;
	
}
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_torqueViscousFriction(JNIEnv *env, jobject obj, jshort id, jdoubleArray jqp)
{
	jdouble *cqp;
	jdouble *ctorque;
	
	ColumnVector qp = ColumnVector(dof[id]);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	qp << cqp;
		
	ColumnVector torque = robot[id].torqueViscousFriction(qp);
	ctorque = torque.data();
//cout << torque << endl;	
	
	jdoubleArray jtorque = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jtorque, 0, dof[id], ctorque);
	return jtorque;
}


JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_torqueCoulombFriction(JNIEnv *env, jobject obj, jshort id, jdoubleArray jqp)
{
	jdouble *cqp;
	jdouble *ctorque;
	
	ColumnVector qp = ColumnVector(dof[id]);
	cqp = env->GetDoubleArrayElements(jqp, NULL);
	qp << cqp;
		
	ColumnVector torque = robot[id].torqueCoulombFriction(qp);
	ctorque = torque.data();
//cout << torque << endl;	
	
	jdoubleArray jtorque = env->NewDoubleArray(dof[id]);
	env->SetDoubleArrayRegion(jtorque, 0, dof[id], ctorque);
	return jtorque;
}
  
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_inertia(JNIEnv *env, jobject obj, jshort id, jdoubleArray jq)
{
	jdouble *cq;
	jdouble *cinertia;
	ColumnVector q = ColumnVector(dof[id]);
	
	cq = env->GetDoubleArrayElements(jq, NULL);
	q << cq;
	
	Matrix inertia = robot[id].inertia(q);
	int size = inertia.size();
	cinertia = inertia.data();
//cout << inertia << endl;	

	jdoubleArray jinertia = env->NewDoubleArray(size);
	env->SetDoubleArrayRegion(jinertia, 0, size, cinertia);
	q.Release();
	inertia.Release();
	return jinertia;
}
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_getGravity(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cg;
	ColumnVector g = robot[id].gravity;
	cg = g.data();
	jdoubleArray jg = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(jg, 0, 3, cg);
	return jg;
}

JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_setGravity(JNIEnv *env, jobject obj, jshort id, jdoubleArray jg)
{
	jdouble *cg;
	cg = env->GetDoubleArrayElements(jg, NULL);
	robot[id].gravity << cg;
}

//-----------------------  CLIK ------------------------------------
/*
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createCLIK__(JNIEnv *env, jobject obj, jshort id)
{
	clik = Clik();
}


JNIEXPORT void JNICALL
Java_javax_robotics_engine_JRoboOp_createCLIK__S_3D_3DD(JNIEnv *env, jobject obj, jshort id, jdoubleArray jKp, jdoubleArray jKo, jdouble jdt)
{
	jdouble *cKp, *cKo;
	DiagonalMatrix Kp = DiagonalMatrix(3);
	DiagonalMatrix Ko = DiagonalMatrix(3);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	cKo = env->GetDoubleArrayElements(jKo, NULL);
	Ko << cKo;
//cout << "\nKp\n" << Kp << "\nKo\n" << Ko << endl;	
	clik[id] =  Clik(robot[id], Kp, Ko, 0.04, 0.04, jdt);
	Kp.Release();
	Ko.Release();
	return;
}
JNIEXPORT void JNICALL
Java_javax_robotics_engine_JRoboOp_createCLIK__S_3D_3DDD(JNIEnv *env, jobject obj, jshort id, jdoubleArray jKp, jdoubleArray jKo, jdouble jeps, jdouble jlambda, jdouble jdt)
{
	jdouble *cKp, *cKo;
	DiagonalMatrix Kp = DiagonalMatrix(3);
	DiagonalMatrix Ko = DiagonalMatrix(3);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	cKo = env->GetDoubleArrayElements(jKo, NULL);
	Ko << cKo;
//cout << "\nKp\n" << Kp << "\nKo\n" << Ko << endl;	
	clik[id] =  Clik(robot[id], Kp, Ko, jeps, jlambda, jdt);
	Kp.Release();
	Ko.Release();
	return;
}

JNIEXPORT void JNICALL
Java_javax_robotics_engine_JRoboOp_createCLIK__S_3DDI(JNIEnv *env, jobject obj, jshort id, jdoubleArray jKp, jdouble jdt, jint jalgorithm)
{
	jdouble *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(env->GetArrayLength(jKp));
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
//cout << "\nKp\n" << Kp << "\nKo\n" << Ko << endl;	
	clik[id] =  Clik(robot[id], Kp, jdt, jalgorithm);
	Kp.Release();
	return;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_qQdot__S_3D_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jpd, jdoubleArray jpddot)
{
	jdouble *cq_qp;
	jdouble *cpd, *cpddot;
	ColumnVector q, qp;
	Matrix q_qp = Matrix(dof[id], 2);
	// posizione desiderata
	cpd = env->GetDoubleArrayElements(jpd, NULL);
	// velocit desiderata
	cpddot = env->GetDoubleArrayElements(jpddot, NULL);
	
	int dim = env->GetArrayLength(jpd);
//cout << "\ndim\n" << dim << endl;
	ColumnVector pd = ColumnVector(dim);
	ColumnVector pddot = ColumnVector(dim);
	pd << cpd;
	pddot << cpddot;
//cout << "\npd\n" << pd << "\npdot\n" << pddot << endl;	
	clik[id].q_qdot(pd, pddot, q, qp);
//cout << "\nq\n" << q << "\nqp\n" << qp << endl;	
        q_qp = q | qp;
	// variabili di ritorno
	cq_qp = q_qp.data();
	jdoubleArray jq_qp = env->NewDoubleArray(2*dof[id]);
	env->SetDoubleArrayRegion(jq_qp, 0, 2*dof[id], cq_qp);
	pd.Release();
	pddot.Release();
	q.Release();
	qp.Release();
	q_qp.Release();
	return jq_qp;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_qQdot__S_3D_3D_3D_3D(JNIEnv *env, jobject obj, jshort id, jdoubleArray jquatd, jdoubleArray jpd, jdoubleArray jpddot, jdoubleArray jwd)
{
	jdouble *cq_qp;
	jdouble *cquatd, *cpd, *cpddot, *cwd;
	ColumnVector pd = ColumnVector(3);
	ColumnVector pddot = ColumnVector(3);
	ColumnVector wd = ColumnVector(3);
	ColumnVector q, qp;
	Matrix q_qp = Matrix(dof[id], 2);
	// quaternione desiderato
	cquatd = env->GetDoubleArrayElements(jquatd, NULL);
	// posizione desiderata
	cpd = env->GetDoubleArrayElements(jpd, NULL);
	// velocit desiderata
	cpddot = env->GetDoubleArrayElements(jpddot, NULL);
	// velocit angolare desiderata
	cwd = env->GetDoubleArrayElements(jwd, NULL);
	
	pd << cpd;
	pddot << cpddot;
	wd << cwd;
	Quaternion quatd = Quaternion(cquatd[0], cquatd[1], cquatd[2], cquatd[3]);
//cout << "\nwd\n" << wd << "\npddot\n" << pddot << "\npd:\n" << pd << "\nquatd:\n" << quatd.s() << endl << quatd.v() << endl;
	clik[id].q_qdot(quatd, pd, pddot, wd, q, qp);
//cout << "\nq\n" << q << "\nqp\n" << qp << endl;	
        q_qp = q | qp;
	// variabili di ritorno
	cq_qp = q_qp.data();
	jdoubleArray jq_qp = env->NewDoubleArray(2*dof[id]);
	env->SetDoubleArrayRegion(jq_qp, 0, 2*dof[id], cq_qp);
	pd.Release();
	pddot.Release();
	wd.Release();
	q.Release();
	qp.Release();
	q_qp.Release();
	return jq_qp;
}
*/
//-----------------------  Spline ------------------------------------
/*
JNIEXPORT double JNICALL 
Java_javax_robotics_engine_JRoboOp_createSplinePath__SLjava_lang_String_2(JNIEnv *env, jobject obj, jshort id, jstring jfileName)
{
	char cfileName[50];
	int fileNameLen = env->GetStringLength(jfileName);
	env->GetStringUTFRegion(jfileName, 0, fileNameLen, cfileName);
	splPath[id] =  Spl_path(cfileName);
	return splPath[id].get_final_time();
}
*/
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createSplinePath(JNIEnv *env, jobject obj, jshort id, jint rowDim, jint nPoints, jdoubleArray jpoints)
{
	jdouble *cpoints;
	Matrix points = Matrix(rowDim, nPoints);
	cpoints = env->GetDoubleArrayElements(jpoints, NULL);
	points << cpoints;
//cout << "\nposition\n" << points << endl;
	splPath[id] =  Spl_path(points);
	return;
}

JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_interpolate(JNIEnv * env, jobject obj, jshort id, jdouble jt)
{
	 jdouble *cp_dp_ddp;
	 ColumnVector p, dp, ddp;
	 
	 jerror[id] = splPath[id].p_pdot_pddot(jt, p, dp, ddp);
	 
         jint dim = p.Nrows();	 
	 jint size = dim*3;
//cout << "\ndim" << dim << "\np\n" << p << "\ndp\n" << dp << "\nddp\n" << ddp << endl;
	 ColumnVector p_dp_ddp = ColumnVector(size);
	 //p_dp_ddp = p & dp & ddp;
	 p_dp_ddp = p & dp & ddp;
	 cp_dp_ddp = p_dp_ddp.data();
	 jdoubleArray jp_dp_ddp = env->NewDoubleArray(size);
	 env->SetDoubleArrayRegion(jp_dp_ddp, 0, size, cp_dp_ddp);
	 return jp_dp_ddp;
}
/*
JNIEXPORT double JNICALL 
Java_javax_robotics_engine_JRoboOp_createSplineQuaternion__SLjava_lang_String_2(JNIEnv *env, jobject obj, jshort id, jstring jfileName)
{
	char cfileName[50];
	int fileNameLen = env->GetStringLength(jfileName);
	env->GetStringUTFRegion(jfileName, 0, fileNameLen, cfileName);
	splQuat[id] =  Spl_Quaternion(cfileName);
	return splQuat[id].get_final_time();
}
*/
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createSplineQuaternion(JNIEnv *env, jobject obj, jshort id, jint nPoints, jdoubleArray jpoints)
{
	quat_map quat_data;
	jdouble *cpoints;
	Matrix points = Matrix(5, nPoints);
	cpoints = env->GetDoubleArrayElements(jpoints, NULL);
	points << cpoints;
//cout << "\norien\n" << points << endl;
	double time, s, vx, vy, vz;
	for(int j=1; j<=nPoints; j++)
	{
	          time = points(1,j);
		  s = points(2, j);
		  vx = points(3, j);
		  vy = points(4, j);
		  vz = points(5, j);
		  Quaternion q = Quaternion(s, vx, vy, vz);
		  quat_data.insert(quat_map::value_type(time, q));
		
	}
	splQuat[id] =  Spl_Quaternion(quat_data);
	
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_interpolateQuat(JNIEnv *env, jobject obj, jshort id, jdouble jt)
{
	 jdouble *cquat_w;
	 ColumnVector wd,  v;
	 ColumnVector quat_w = ColumnVector(7);
	 Quaternion q;
	 
	 jerror[id] =  splQuat[id].quat_w(jt, q, wd);
//cout <<  "wd\n" << wd << endl; 
	 v = q.v();
//cout << "v\n" << v << endl;
	 quat_w << q.s() << v(1) << v(2) << v(3) << wd(1) << wd(2) << wd(3);
	 cquat_w = quat_w.data();
//cout << "quat_w\n" <<  quat_w << endl; 
	 jdoubleArray jquat_w = env->NewDoubleArray(7);
	 env->SetDoubleArrayRegion(jquat_w, 0, 7, cquat_w);
	 return jquat_w;
	
}
JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_interpolatePathQuat(JNIEnv *env, jobject obj, jshort id, jdouble jt)
{
	 jdouble *cp_dp_ddp_quat_w;
	 ColumnVector p, dp, ddp, wd, v;
	 Quaternion q;
	 
	 jerror[id] = splPath[id].p_pdot_pddot(jt, p, dp, ddp);
	 jerror[id] =  splQuat[id].quat_w(jt, q, wd); 
	 jint size = 3+3+3+4+3;
	 ColumnVector p_dp_ddp_quat_w = ColumnVector(size);
         v = q.v();
         p_dp_ddp_quat_w << p(1) << p(2) << p(3) << dp(1) << dp(2) << dp(3) << ddp(1) << ddp(2) << ddp(3) << q.s() << v(1) << v(2) << v(3) << wd(1) << wd(2) << wd(3);
	 cp_dp_ddp_quat_w = p_dp_ddp_quat_w.data();
	 jdoubleArray jp_dp_ddp_quat_w = env->NewDoubleArray(size);
	 env->SetDoubleArrayRegion(jp_dp_ddp_quat_w, 0, size, cp_dp_ddp_quat_w);
	 return jp_dp_ddp_quat_w;
}

//-----------------------  Controllers ------------------------------------

/* Proportional derivative */
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createProportionalDerivative(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp, jdoubleArray jKd)
{
	jdouble *cKd, *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	pd[id] = Proportional_Derivative(robot[robotID], Kp, Kd);
	
	Kd.Release();
	Kp.Release();
	return;
}
JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKpPD(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp)
{
	jdouble *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	jerror[id] = pd[id].set_Kp(Kp);
	Kp.Release();
        return jerror[id];
}


JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKdPD(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKd)
{
	jdouble *cKd;
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	jerror[id] = pd[id].set_Kd(Kd);
	Kd.Release();
        return jerror[id];
}

JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdPD(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jqd, jdoubleArray jqpd)
{
	 jdouble *ctorque;
	 jdouble *cqd, *cqpd;
	 ColumnVector qd = ColumnVector(dof[robotID]);
	 ColumnVector qpd = ColumnVector(dof[robotID]);
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cqd = env->GetDoubleArrayElements(jqd, NULL);
	 qd << cqd;
	 cqpd = env->GetDoubleArrayElements(jqpd, NULL);
	 qpd << cqpd;
 
	 torque = pd[id].torque_cmd(robot[robotID], qd, qpd);
//cout << setw(8) << setprecision(4) << (qd.t() | qpd.t() | torque.t()) << endl;		 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 qd.Release();
	 qpd.Release();
	 torque.Release();
	 return jtorque;
		
}
/* Proportional derivative with gravity compensation */
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createPDGravityComp(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp, jdoubleArray jKd)
{
	jdouble *cKd, *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	pdg[id] = PDGravityComp(robot[robotID], Kp, Kd);
	
	Kd.Release();
	Kp.Release();
	return;
}

JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdPDG(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jqd)
{
	 jdouble *ctorque;
	 jdouble *cqd;
	 ColumnVector qd = ColumnVector(dof[robotID]);
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cqd = env->GetDoubleArrayElements(jqd, NULL);
	 qd << cqd;
	 
	 torque = pdg[id].torque_cmd(robot[robotID], qd);
//cout << setw(8) << setprecision(4) << (qd.t()  | torque.t()) << endl;		 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 qd.Release();
	 torque.Release();
	 return jtorque;
}

JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKpPDG(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp)
{
	jdouble *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	jerror[id] = pdg[id].set_Kp(Kp);
        return jerror[id];
}

JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKdPDG(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKd)
{
	jdouble *cKd;
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	jerror[id] = pdg[id].set_Kd(Kd);
	return jerror[id];
}

/* Computed Torque Method */
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createComputedTorqueMethod(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp, jdoubleArray jKd)
{
	jdouble *cKd, *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	ctm[id] = Computed_torque_method(robot[robotID], Kp, Kd);
	return;
}
JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKpCTM(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp)
{
	jdouble *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	jerror[id] = ctm[id].set_Kp(Kp);
        return jerror[id];
}


JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKdCTM(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKd)
{
	jdouble *cKd;
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	jerror[id] = ctm[id].set_Kd(Kd);
        return jerror[id];
}
JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdCTM(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jqd, jdoubleArray jqpd, jdoubleArray jqppd)
{
	 jdouble *ctorque;
	 jdouble *cqd, *cqpd, *cqppd;
	 ColumnVector qd = ColumnVector(dof[robotID]);
	 ColumnVector qpd = ColumnVector(dof[robotID]);
         ColumnVector qppd = ColumnVector(dof[robotID]);
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cqd = env->GetDoubleArrayElements(jqd, NULL);
	 qd << cqd;
	 cqpd = env->GetDoubleArrayElements(jqpd, NULL);
	 qpd << cqpd;

	 cqppd  = env->GetDoubleArrayElements(jqppd, NULL);
	 qppd << cqppd;
 
	 torque = ctm[id].torque_cmd(robot[robotID], qd, qpd, qppd);
// cout << torque << endl;		 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 return jtorque;
		
}
/* Robust control */
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createRobustControl(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp, jdoubleArray jKd, jdoubleArray jB, jdoubleArray jQ, jdouble rho, jdouble eps)
{
	jdouble *cKd, *cKp, *cB, *cQ;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	Matrix B = SquareMatrix(dof[robotID]);
	Matrix Q = SquareMatrix(2*dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
//cout << "\nKp\n" << Kp << endl;
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
//cout << "\nKd\n" << Kd << endl;
        cB = env->GetDoubleArrayElements(jB, NULL);
	B << cB;
//cout << "\nB\n" << B << endl;
	cQ = env->GetDoubleArrayElements(jQ, NULL);
	Q << cQ;
//cout << "\nQ\n" << Q << endl;
	robust[id] = Robust_control(robot[robotID], Kp, Kd, B, Q, rho, eps);
	
	return;
}

JNIEXPORT jdoubleArray JNICALL 
Java_javax_robotics_engine_JRoboOp_torqueCmdRC(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jqd, jdoubleArray jqpd, jdoubleArray jqppd)
{
	 jdouble *ctorque;
	 jdouble *cqd, *cqpd, *cqppd;
	 ColumnVector qd = ColumnVector(dof[robotID]);
	 ColumnVector qpd = ColumnVector(dof[robotID]);
         ColumnVector qppd = ColumnVector(dof[robotID]);
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cqd = env->GetDoubleArrayElements(jqd, NULL);
	 qd << cqd;
	 cqpd = env->GetDoubleArrayElements(jqpd, NULL);
	 qpd << cqpd;

	 cqppd  = env->GetDoubleArrayElements(jqppd, NULL);
	 qppd << cqppd;
 
	 torque = robust[id].torque_cmd(robot[robotID], qd, qpd, qppd);
// cout << torque << endl;		 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 return jtorque;
		
}

JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKpRC(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKp)
{
	jdouble *cKp;
	DiagonalMatrix Kp = DiagonalMatrix(dof[robotID]);
	cKp = env->GetDoubleArrayElements(jKp, NULL);
	Kp << cKp;
	jerror[id] = robust[id].set_Kp(Kp);
        return jerror[id];
	
}

JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_setKdRC(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jKd)
{
	jdouble *cKd;
	DiagonalMatrix Kd = DiagonalMatrix(dof[robotID]);
	cKd = env->GetDoubleArrayElements(jKd, NULL);
	Kd << cKd;
	jerror[id] = robust[id].set_Kd(Kd);
	return jerror[id];
	
}
/* Resolved Rate Acceleration */
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createResolveRateAcceleration(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdouble kvp, jdouble kpp, jdouble kvo, jdouble kpo)
{
	ra[id] = Resolved_acc(robot[robotID], kvp, kpp, kvo, kpo);
	return;
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_getPosErrorRRA(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cerr;
	ColumnVector err = ra[id].get_pos_error();
	cerr = err.data();
	jdoubleArray jerr = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(jerr, 0, 3, cerr);
	return jerr;
	
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_getQuatErrorRRA(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cerr;
	ColumnVector err = ra[id].get_quat_error();
	cerr = err.data();
	jdoubleArray jerr = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(jerr, 0, 3, cerr);
	return jerr;
}

JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdRRA__SS_3D_3D_3D_3D_3D_3DS(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jpdpp, jdoubleArray jpdp, jdoubleArray jpd, jdoubleArray jwdp, jdoubleArray jwd, jdoubleArray jqd, jshort linkPc)
{
	 jdouble *ctorque;
	 jdouble *cpdpp, *cpdp, *cpd, *cwdp, *cwd, *cqd;
	 ColumnVector pdpp = ColumnVector(3);
	 ColumnVector pdp = ColumnVector(3);
	 ColumnVector pd = ColumnVector(3);
	 ColumnVector wdp = ColumnVector(3);
	 ColumnVector wd = ColumnVector(3);
	 Quaternion qd; 
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cpdpp = env->GetDoubleArrayElements(jpdpp, NULL);
	 pdpp << cpdpp;
	 
	 cpdp = env->GetDoubleArrayElements(jpdp, NULL);
	 pdp << cpdp;
	 
	 cpd = env->GetDoubleArrayElements(jpd, NULL);
	 pd << cpd;
	 
	 cwd = env->GetDoubleArrayElements(jwdp, NULL);
	 wdp << cwdp;
	 
	 cwd = env->GetDoubleArrayElements(jwd, NULL);
	 wd << cwd;
	 
	 cqd = env->GetDoubleArrayElements(jqd, NULL);
	 qd = Quaternion(cqd[0], cqd[1], cqd[2], cqd[3]);
	 
	 torque = ra[id].torque_cmd(robot[robotID], pdpp, pdp, pd, wdp, wd, qd, linkPc, 0.0);
	 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 pdpp.Release();
	 pdp.Release();
	 pd.Release();
	 wdp.Release();
	 wd.Release();
	 torque.Release();
	 return jtorque;
	
}
JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdRRA__SS_3D_3D_3D(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jpdpp, jdoubleArray jpdp, jdoubleArray jpd)
{
	 jdouble *ctorque;
	 jdouble *cpdpp, *cpdp, *cpd;
	 ColumnVector pdpp = ColumnVector(3);
	 ColumnVector pdp = ColumnVector(3);
	 ColumnVector pd = ColumnVector(3);
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cpdpp = env->GetDoubleArrayElements(jpdpp, NULL);
	 pdpp << cpdpp;
	 
	 cpdp = env->GetDoubleArrayElements(jpdp, NULL);
	 pdp << cpdp;
	 
	 cpd = env->GetDoubleArrayElements(jpd, NULL);
	 pd << cpd;
//cout << "\njpdp\n" << pdp << "\njpdpp\n" << pdpp << endl;
	 torque = ra[id].torque_cmd(robot[robotID], pdpp, pdp, pd);
	 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 return jtorque;
}
/* Gravity Compensation */
JNIEXPORT void 
JNICALL Java_javax_robotics_engine_JRoboOp_createGravityCompensation(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdouble kvp, jdouble kpp, jdouble kvo, jdouble kpo)
{
	gcomp[id] = Gravity_comp(robot[robotID], kvp, kpp, kvo, kpo);
	return;
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_getPosErrorGCOMP(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cerr;
	ColumnVector err = gcomp[id].get_pos_error();
	cerr = err.data();
	jdoubleArray jerr = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(jerr, 0, 3, cerr);
	return jerr;
	
}

JNIEXPORT jdoubleArray JNICALL
Java_javax_robotics_engine_JRoboOp_getQuatErrorGCOMP(JNIEnv *env, jobject obj, jshort id)
{
	jdouble *cerr;
	ColumnVector err = gcomp[id].get_quat_error();
	cerr = err.data();
	jdoubleArray jerr = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(jerr, 0, 3, cerr);
	return jerr;
}

JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdGCOMP__SS_3D_3DS(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jpd, jdoubleArray jqd, jshort linkPc)
{
         jdouble *ctorque;
	 jdouble *cpd, *cwdp, *cqd;
	 ColumnVector pd = ColumnVector(3);
	 Quaternion qd; 
	 ColumnVector torque = ColumnVector(dof[robotID]);
	
	 cpd = env->GetDoubleArrayElements(jpd, NULL);
	 pd << cpd;
	 
	 cqd = env->GetDoubleArrayElements(jqd, NULL);
	 qd = Quaternion(cqd[0], cqd[1], cqd[2], cqd[3]);
	 
	 torque = gcomp[id].torque_cmd(robot[robotID], pd, qd, linkPc, 0.0);
	 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 return jtorque;
}

JNIEXPORT jdoubleArray 
JNICALL Java_javax_robotics_engine_JRoboOp_torqueCmdGCOMP__SS_3D(JNIEnv *env, jobject obj, jshort id, jshort robotID, jdoubleArray jpd)
{
	 jdouble *ctorque;
	 jdouble *cpd;
	 ColumnVector pd = ColumnVector(3);
	 ColumnVector torque = ColumnVector(dof[robotID]);
	 
	 cpd = env->GetDoubleArrayElements(jpd, NULL);
	 pd << cpd;
	 torque = gcomp[id].torque_cmd(robot[robotID], pd);
	 
	 ctorque = torque.data();
	 jdoubleArray jtorque = env->NewDoubleArray(dof[robotID]);
	 env->SetDoubleArrayRegion(jtorque, 0, dof[robotID], ctorque);
	 
	 return jtorque;
	
}

/* Impedance control TODO*/
JNIEXPORT void JNICALL 
Java_javax_robotics_engine_JRoboOp_createImpedance(JNIEnv *, jobject, jshort, jshort, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray)
{
	
}

JNIEXPORT jshort JNICALL 
Java_javax_robotics_engine_JRoboOp_impedanceControl(JNIEnv *, jobject, jshort, jshort, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdoubleArray, jdouble)
{
	return 0;
}
