/*
 * EndEffController.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.controllers;

import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;


/**
 * <p>
 * Interface for different schemes of robot end effector position and orientation controller.  
 * </p>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * @version 7/10/2005
 */
public interface EndEffController extends Controller
{
    
    /**
     * Gets the position error.
     * 
     * @return the position error vector.
     */
    public RVector3d getPositionError();
    
    /**
     * Gets the current orientation error.
     * The error is computed form quaternion as:
     * <p>
     * &eta;(q)&epsilon<sub>d</sub> - &eta<sub>d</sub>&epsilon(q) -
     * S(&epsilon<sub>d</sub>)&epsilon(q)
     * </p> where &eta; and &epsilon; are the vector and the scalar components of quaternion respectively.
     * @return the orientation error vector.
     */
    public RVector3d getOrientationError();
    
    
    /**
     * Sets the output torque for a desired end effector: acceleration, velocity, position, angular acceleration, angular velocity, angular position.
     * 
     * @param pdpp desired acceleration
     * @param pdp desired velocity
     * @param pd desired position
     * @param wdp desired angular acceleration
     * @param wd desired angular velocity
     * @param quatd desired vector angular position: quaternion (s,v1,v2,v3)
     * @return the output torque
     */
    public RVector torqueCmd(RVector3d pdpp, RVector3d pdp, RVector3d pd, RVector3d wdp, RVector3d wd, Quaternion quatd);
  
    /**
     * Sets the output torque for a desired end effector: acceleration, velocity, position.
     * 
     * @param pdpp desired acceleration
     * @param pdp desired velocity
     * @param pd desired position
     * @return the output torque
     */
    public RVector torqueCmd(RVector3d pdpp, RVector3d pdp, RVector3d pd);
    
    
}    
    
    