package javax.robotics.image;

/*
 *	@(#)ImageDisplayer.java 1.5 02/04/01 15:04:10
 *
 * Copyright (c) 1996-2002 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN
 * OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR
 * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that Software is not designed,licensed or intended
 * for use in the design, construction, operation or maintenance of
 * any nuclear facility.
 */

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;

/**
 * Utility class to display buffered images.
 */
public final class ImageDisplayer extends JFrame implements ActionListener
{

    /**
     * 
     */
    private static final long serialVersionUID = 3257002163870970164L;
    BufferedImage bImage;
    final String title;
   
    public ImageDisplayer(BufferedImage bImage, String title)
    {

        this.bImage = bImage;
        this.title = title;
        this.setTitle(title);

        // Create and initialize menu bar
        this.setJMenuBar(createMenuBar());

        // Create scroll pane, and embedded image panel
        ImagePanel imagePanel = new ImagePanel();
        JScrollPane scrollPane = new JScrollPane(imagePanel);
        scrollPane.getViewport().setPreferredSize(new Dimension(600, 600));

        // Add scroll pane to the frame and make it visible
        this.getContentPane().add(scrollPane);
        this.pack();
        this.setVisible(true);
    }

    class ImagePanel extends JPanel
    {
        /**
         * 
         */
        static final long serialVersionUID = 4050768195382228275L;

        public void paint(Graphics g)
        {
            g.setColor(Color.black);
            g.fillRect(0, 0, getSize().width, getSize().height);
            g.drawImage(bImage, 0, 0, this);
        }

        private ImagePanel()
        {
            setPreferredSize(new Dimension(bImage.getWidth(), bImage
                    .getHeight()));
        }
    }

    private JMenuItem printItem;
    private JMenuItem closeItem;
    private JMenuItem saveItem;

    public void actionPerformed(ActionEvent event)
    {
        final Object target = event.getSource();

        if(target.equals(printItem))
        {
            new ImagePrinter(bImage).print();
            return;
        }

        if(target.equals(saveItem))
        {
            try
            {
                final ImageFileChooser fc = new ImageFileChooser();
                final String fileName = fc.setFileName(title + ".png");
                ImageIO.write(bImage, "png", new File(fileName));
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            return;
        }

        if(target.equals(closeItem))
        {
            this.removeAll();
            this.setVisible(false);
            bImage = null;
        }
    }

    private final JMenuBar createMenuBar()
    {
        final JMenuBar menuBar = new JMenuBar();
        final JMenu fileMenu = new JMenu("File");
        saveItem = new JMenuItem("Save");
        saveItem.addActionListener(this);
        printItem = new JMenuItem("Print");
        printItem.addActionListener(this);
        closeItem = new JMenuItem("Close");
        closeItem.addActionListener(this);
        fileMenu.add(printItem);
        fileMenu.add(new JSeparator());
        fileMenu.add(saveItem);
        fileMenu.add(new JSeparator());
        fileMenu.add(closeItem);
        menuBar.add(fileMenu);
        return menuBar;
    }

}
