/*
 * ImageFileChooser.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.image;

import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileSystemView;

/**
 * Dialog panel to select xml configuration file.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 * @version 5/10/2005
 */
public class ImageFileChooser extends JFileChooser
{

    private ImageFileFilter filter;
    /**
     * 
     */
    private static final long serialVersionUID = 3762530101039739442L;

    /**
     * Constructs a ImageFileChooser pointing to the user's default directory.
     */
    public ImageFileChooser()
    {
        super();
        setFilter();
    }

    /**
     * Constructs a ImageFileChooser using the given path.
     * @param currentDirectoryPath the path
     */
    public ImageFileChooser(String currentDirectoryPath)
    {
        super(currentDirectoryPath);
        setFilter();
    }

    /**
     * Constructs a ImageFileChooser using the given File as the path.
     * @param currentDirectory the path
     */
    public ImageFileChooser(File currentDirectory)
    {
        super(currentDirectory);
        setFilter();
    }

    /**
     * Constructs a ImageFileChooser using the given current directory and FileSystemView.
     * @param currentDirectory the path
     * @param fsv the FileSystemView
     */
    public ImageFileChooser(File currentDirectory, FileSystemView fsv)
    {
        super(currentDirectory, fsv);
        setFilter();
    }

    /**
     * Constructs a ImageFileChooser using the given current directory and FileSystemView.
     * @param currentDirectoryPath the path
     * @param fsv the FileSystemView
     */
    public ImageFileChooser(String currentDirectoryPath, FileSystemView fsv)
    {
        super(currentDirectoryPath, fsv);
        setFilter();
    }

    /**
     * Gets the canonical name of the choosen file.
     * @return the file name with full path.
     * @throws IOException
     */
    public String getFileName() throws IOException
    {

        final int returnVal = this.showOpenDialog(null);
        final String fileName;
        fileName = (returnVal == JFileChooser.APPROVE_OPTION)? this.getSelectedFile().getCanonicalPath():null;
        return fileName;

    }
    
    /**
     * Gets the canonical name of the choosen file.
     * @return the file name with full path.
     * @throws IOException
     */
    public String setFileName(String name) throws IOException
    {
        this.setSelectedFile(new File(name));
        final int returnVal = this.showSaveDialog(null);
        final String fileName;
        fileName = (returnVal == JFileChooser.APPROVE_OPTION)? this.getSelectedFile().getCanonicalPath():name;
        return fileName;
   }
    private void setFilter()
    {
        filter = new ImageFileFilter();
        this.setFileFilter(filter);

    }

}
