/*
 * Robot3DSceneGraph.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.j3d;

import javax.media.j3d.Background;
import javax.media.j3d.Behavior;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;

/**
 * Creates a default scene 3D with axes and lights.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 04/11/2005
 */
public class Robot3DSceneGraph extends BranchGroup
{
    private final Background background;
    
    public Robot3DSceneGraph()
    {
        this(null);
    }
    /**
     * Creates a default scene 3D with axes and lights
     * @param bh the Behavior class to add.
     */
    public Robot3DSceneGraph(Behavior bh)
    {
        super();
        final BoundingSphere bounds = new BoundingSphere(new Point3d(), 1000.0);
        background = new Background(new Color3f(0.2f, 0.3f, 0.50f));
        background.setApplicationBounds(bounds);
        background.setCapability(Background.ALLOW_COLOR_WRITE);
        background.setCapability(Background.ALLOW_COLOR_READ);
        
        // ramo principale
        setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        setCapability(BranchGroup.ALLOW_CHILDREN_READ);
       
        /*
         * Transform3D translate = new Transform3D(); transformVp = new
         * Transform3D(); vpTrans = new TransformGroup(); // View Platform
         * vpTrans = simpleU.getViewingPlatform().getViewPlatformTransform();
         * transformVp.setTranslation(new Vector3d(0.0, 0.0, 1.80));
         * vpTrans.setTransform(transformVp);
         */
        // attacco del ramo del manipolatore
        // attacco della terna 0
        addChild(new Axes(0.8f, true));
               
        // ------------------Impostazione delle luci
        addChild(new Lights());

        // impostazione behavior
        if(bh != null)
        {
            bh.setSchedulingBounds(bounds);
            addChild(bh);
        }
        // Impostazione dello sfondo
        addChild(background);
   }
   
    /**
     * Gets the child background.
     * @return the Background class
     */
    public final Background getBackground()
    {
        return background;
    }
}
