/*
 * Heun.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.ode;

import javax.robotics.vecmath.RVector;



/**
 * <p>
 * This class implements Heun integration method.  
 * </p>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * @version 09/11/2005
 */
public class Heun extends Euler 
{
   
    private int size;
    
    /**
     * Defines numerical integration for x' = f(x) differential equation.
     * @param dt time interval of integration [sec]
     * @param initialCondition initial condition at time zero x(0).
     * @param target differential equation to be integrated
     */
    public Heun(double dt, RVector initialCondition, DerivableFunction target)
    {
        super(dt, initialCondition, target);
        size = initialCondition.getSize();
    }
    
  
   
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#step(RVector)
     */
    public void step(RVector x)
    {
      final  RVector xdot1, xdot;
      final RVector x1 = new RVector(size);
      
      // x' = f(x)
      xdot = xDot(x);
      
      x1.scaleAdd(h, xdot, lastValue);
      
      // f(x+h*f(x))
       xdot1 = xDot(x1);
      
      xdot.add(xdot1);
      lastValue.scaleAdd(h*0.5, xdot, lastValue);
      
      time += h;
    }   
}
