/*
 * CLIKSimulation.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.simulator;

import java.util.Enumeration;

import javax.robotics.engine.clik.CLIKR;
import javax.robotics.engine.clik.ConstraintVelocity;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;

public class ClikWithConstraintSimulation extends CLIKRSimulation
{

    public ClikWithConstraintSimulation(Robot3D robot, CLIKR clik,
            ConstraintVelocity constraint, Trajectory trj, int points)
    {
        super(robot, clik, constraint, trj, points);
    }

    @Override
    public void initialize()
    {
        robot.setQ3D(q);
        // end effector path
        trj.interpolate(time, s, ds, dds);

        // clik
        clik.step(s, ds, constraint);

        q.get(jointData[i]);

        wakeupOn(wakeUp);

        sceneTimer.scheduleAtFixedRate(new Update3DScene(), 0l,
                updateSchedulerTime);

        createPlotPanel();
        updatePlotPanel();

    }

    @Override
    public void processStimulus(Enumeration arg0)
    {
        if(i >= points - 1)
        {
            this.setEnable(false);
            sceneTimer.cancel();
            plotPanel.fillPlots();
            return;
        }

        time += timeStep;
        wakeupOn(wakeUp);

        // end effector path
        trj.interpolate(time, s, ds, dds);

        clik.step(s, ds, constraint);

        i++;
        q.get(jointData[i]);

        updatePlotPanel();
    }

}
