/*
 * DisplayPanel.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;

import java.awt.Font;
import java.awt.HeadlessException;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
/**
 * This class creates a panel to display output information.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 29/10/2005
 */
public final class DisplayPanel extends JFrame
{
    private final JTextArea textArea;
    /**
     * 
     */
    private static final long serialVersionUID = -5843765437081989460L;

    /**
     * Constructor of sizeable window with texta area.
     * @param title the title of the window.
     * @throws HeadlessException
     */
    public DisplayPanel(String title) throws HeadlessException
    {
        super(title);
        textArea = new JTextArea();
        textArea.setFont(new Font("monospaced", Font.PLAIN, 12));
        textArea.setLineWrap(true);
        textArea.setWrapStyleWord(true);
        textArea.setEditable(false);
        final JScrollPane areaScrollPane = new JScrollPane(textArea);
        areaScrollPane
                .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
       // areaScrollPane.setPreferredSize(new Dimension(250, 250));
        
        setSize(500, 450);
        add(areaScrollPane);
        setVisible(true);
    }
    
    /**
     * Display the input text in the window. 
     * @param text the text to be displayed
     */
    public void setText(String text)
    {
        textArea.setText(text);
    }
    /**
     * Append the input text in the window. 
     * @param text the text to be displayed
     */
    
    public void append(String text)
    {
        textArea.append(text);
    }
}
