/*
 * GravityPanel.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.RVector3d;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * This class creates a panel to set the gravity vector.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 */
public class GravityPanel extends JPanel implements ChangeListener
{
    private Robot robot;
    private RVector3d gravity;
    JButton ng, bx, by, bz;
    JSpinner x, y, z;
    
    /**
     * 
     */
    private static final long serialVersionUID = 3688502186594546484L;

    /**
     * The panel constructor
     * 
     * @param robot
     *            the input robot.
     */
    public GravityPanel(Robot robot)
    {
        super();
        this.robot = robot;
        setBorder(BorderFactory
                .createCompoundBorder(BorderFactory.createTitledBorder(
                        BorderFactory.createLineBorder(Color.gray), "Gravity"),
                        BorderFactory.createEmptyBorder(2, 2, 2, 2)));

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        add(createSpinnerButtonPanel());

        JPanel p = new JPanel();
        ng = new JButton("No gravity");
        ng.addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                x.setValue(new Double(0.0));
                y.setValue(new Double(0.0));
                z.setValue(new Double(0.0));
            }

        });
        p.add(ng);
        add(p);
        setValues();

    }

    private JSpinner createJSpinner(String tip)
    {
        final JSpinner spinner = new JSpinner(new SpinnerNumberModel(0.0, -100.0,
                100.0, 0.1));
        spinner.addChangeListener(this);
        spinner.setToolTipText(tip);
        // System.out.println(spinner.getPreferredSize());
        // spinner.setPreferredSize(new Dimension(50, 20));
        return spinner;
    }

    private JPanel createSpinnerButtonPanel()
    {
        final JPanel p = new JPanel();
        p.setLayout(new GridLayout(2, 3, 0, 0));
        x = createJSpinner("-x");
        y = createJSpinner("-y");
        z = createJSpinner("-z");

        p.add(x);
        p.add(y);
        p.add(z);
        bx = new JButton("x");
        bx.addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                x.setValue(new Double(9.81));
                y.setValue(new Double(0.0));
                z.setValue(new Double(0.0));
            }

        });
        by = new JButton("y");
        by.addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                x.setValue(new Double(0.0));
                y.setValue(new Double(9.81));
                z.setValue(new Double(0.0));
            }

        });
        bz = new JButton("z");
        bz.addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                x.setValue(new Double(0.0));
                y.setValue(new Double(0.0));
                z.setValue(new Double(9.81));

            }

        });
        p.add(bx);
        p.add(by);
        p.add(bz);
        return p;
    }
    
    private void setValues()
    {
        gravity = robot.getGravity();
        x.setValue(new Double(gravity.x));
        y.setValue(new Double(gravity.y));
        z.setValue(new Double(gravity.z));
    }

    public void stateChanged(ChangeEvent e)
    {
        robot.setGravity(new RVector3d((Double) x.getValue(), (Double) y
                .getValue(), (Double) z.getValue()));
    }

}
