/*
 * JacobianPanel.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.Matrix;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;

/**
 * This class creates a panel for robot jacobian calculation.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 18/11/2005
 */
public final class JacobianPanel extends JPanel
{

    /**
     * 
     */
    private static final long serialVersionUID = -3859799291836059064L;
    private static final String strLineSeparator = "line.separator";
    private static final String newLine = System.getProperty(strLineSeparator);
    private final JButton compute;
    private final DisplayPanel display;

    /**
     * The constructor of the panel.
     * 
     * @param robot
     *            the robot class to link to
     */
    public JacobianPanel(final Robot robot)
    {
        super();
        display = new DisplayPanel("Jacobian Matrix");
        display.setVisible(false);
        compute = new JButton("Compute");
       // compute.setSize(100, 30);
        // compute.setFont(new Font("Helvetica", Font.BOLD, 10));
        compute.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                final Matrix J = robot.jacobian();
                final int n = J.getNumCol();
                final int m = J.getNumRow();
                final Matrix Q = new Matrix(n, n);
                final Matrix U = new Matrix(m, n);
                final Matrix V = new Matrix(n, n);
                final Integer rango = new Integer(J.svd(Q, U, V));
                display.setText("Jacobian matrix:" + newLine);
                display.append(J.toString());
                display.append("Singular Value Decomposition U*Q*V^t:"
                        + newLine);
                display.append("U" + newLine + U.toString());
                display.append("Q" + newLine + Q.toString());
                display.append("V" + newLine + V.toString());
                display.append("rank:" + newLine);
                display.append(rango.toString());
                display.setVisible(true);

            }

        });
       
        setBorder(BorderFactory.createCompoundBorder(BorderFactory
                .createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                        "Jacobian"), BorderFactory
                .createEmptyBorder(5, 5, 5, 5)));

        add(compute);
      
    }

}
