/*
 * LookAndFeelSelector.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;

import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;
import com.l2fprod.util.OS;

public final class LookAndFeelSelector
{
//  Possible Look & Feels
    private static final String mac      =
            "com.sun.java.swing.plaf.mac.MacLookAndFeel";
    private static final String metal    =
            "javax.swing.plaf.metal.MetalLookAndFeel";
    private static final String motif    =
            "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    private static final String windows  =
            "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
    private static final String gtk  =
            "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
    
    static public void setJavaDefault() throws SecurityException,
            IllegalArgumentException, NoSuchMethodException,
            IllegalAccessException, InvocationTargetException,
            ClassNotFoundException, InstantiationException, UnsupportedLookAndFeelException
    {
       
            UIManager.setLookAndFeel(metal);
            UIManager.put("swing.boldMetal", Boolean.TRUE);
            setDecoration(false);
       }
    
    static public void setMotif() throws SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, InstantiationException, UnsupportedLookAndFeelException
    {
            UIManager.setLookAndFeel(motif);
            setDecoration(false);
       }
    
    static public void setWindows() throws SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, InstantiationException, UnsupportedLookAndFeelException
    {
            UIManager.setLookAndFeel(windows);
            setDecoration(false);
       }
    
    static public void setMac() throws SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, InstantiationException, UnsupportedLookAndFeelException
    {
            UIManager.setLookAndFeel(mac);
            setDecoration(false);
    }
    
    static public void setGTK() throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException, SecurityException, IllegalArgumentException, NoSuchMethodException, InvocationTargetException
    {
            UIManager.setLookAndFeel(gtk);
            setDecoration(false);
    }
    
    static public void setSkin(InputStream fileTheme) throws SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException
    {
        try
        {
            
            SkinLookAndFeel.setSkin(SkinLookAndFeel.loadThemePack(fileTheme));
            UIManager
                    .setLookAndFeel("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
            
        }
        catch (Exception e)
        {
            
            e.printStackTrace();
        }
        setDecoration(true);
    }
    
    static public void setSkin(URL fileTheme) throws SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException
    {
        try
        {
            
            SkinLookAndFeel.setSkin(SkinLookAndFeel.loadThemePack(fileTheme));
            UIManager
                    .setLookAndFeel("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
            
        }
        catch (Exception e)
        {
            
            e.printStackTrace();
        }
        setDecoration(true);
    }
    
    static public void setSkin(String fileTheme) throws SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException
    {
        try
        {
            
            SkinLookAndFeel.setSkin(SkinLookAndFeel.loadThemePack(fileTheme));
            UIManager
                    .setLookAndFeel("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
            
        }
        catch (Exception e)
        {
            
            e.printStackTrace();
        }
        setDecoration(true);
    }
    
    static private void setDecoration(boolean isSet) throws SecurityException,
            NoSuchMethodException, IllegalArgumentException,
            IllegalAccessException, InvocationTargetException
    {
        // if we are running with JDK1.4 decorates the frames and dialogs
        if(OS.isOneDotFourOrMore())
        {
            java.lang.reflect.Method method = JFrame.class.getMethod(
                    "setDefaultLookAndFeelDecorated", new Class[]
                        { boolean.class });
            method.invoke(null, new Object[]
                { isSet });

            method = JDialog.class.getMethod("setDefaultLookAndFeelDecorated",
                    new Class[]
                        { boolean.class });
            method.invoke(null, new Object[]
                { isSet });
            
        }
    }
}
