/*
 * ImageFileChooser.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.xml;

import java.io.File;
import java.io.IOException;
import java.net.URI;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileSystemView;

/**
 * Dialog panel to select xml configuration file.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 * @version 30/10/2005
 */
public class XmlFileChooser extends JFileChooser
{

    private XmlFileFilter filter;
    /**
     * 
     */
    private static final long serialVersionUID = 3762530101039739442L;

    /**
     * Constructs the chooser pointing to the user's default directory.
     */
    public XmlFileChooser()
    {
        super();
        setXmlFilter();
    }

    /**
     * Constructs the chooser from the given path.
     * @param currentDirectoryPath the path
     */
    public XmlFileChooser(String currentDirectoryPath)
    {
        super(currentDirectoryPath);
        setXmlFilter();
    }

    /**
     * Constructs the chooser from a given File as the path.
     * @param currentDirectory the path
     */
    public XmlFileChooser(File currentDirectory)
    {
        super(currentDirectory);
        setXmlFilter();
    }

    /**
     * Constructs the chooser using the given current directory and FileSystemView.
     * @param currentDirectory the path
     * @param fsv the FileSystemView
     */
    public XmlFileChooser(File currentDirectory, FileSystemView fsv)
    {
        super(currentDirectory, fsv);
        setXmlFilter();
    }

    /**
     * Constructs the chooser using the given current directory and FileSystemView.
     * @param currentDirectoryPath the path
     * @param fsv the FileSystemView
     */
    public XmlFileChooser(String currentDirectoryPath, FileSystemView fsv)
    {
        super(currentDirectoryPath, fsv);
        setXmlFilter();
    }

    /**
     * Gets the canonical name of the choosen file.
     * @return the file name with full path.
     * @throws IOException
     */
    public String getFileName() throws IOException
    {
       
        final String fileName;
        final File file = getFile();
        fileName = (file != null)? file.getCanonicalPath():null;
        return fileName;

    }
    
    /**
     * Gets the file.
     * @return the file.
     * @throws IOException
     */
    public File getFile() throws IOException
    {

        final int returnVal = this.showOpenDialog(null);
        final File fileName;
        fileName = (returnVal == JFileChooser.APPROVE_OPTION)? this.getSelectedFile():null;
        return fileName;

    }
    
    
    /**
     * Gets the file URI.
     * @return the URI of the file.
     * @throws IOException
     */
    public URI getURI() throws IOException 
    {
        final URI uri;
        final File file = getFile();
        uri = (file != null)? file.toURI():null;
        return uri;
    }
    
    private void setXmlFilter()
    {
        filter = new XmlFileFilter();
        this.setFileFilter(filter);

    }

}
